@@ -8,6 +8,7 @@
 #include <qsqldatabase.h>
 #include <qurl.h>
 #include <qnetwork.h>
+#include <qwaitcondition.h>
 
 #include <qsocketdevice.h>
 #include <qhostaddress.h>
@@ -15,17 +16,29 @@
 #include <cmath>
 #include <queue>
 
+#include "config.h"
 #include "mythcontext.h"
 #include "oldsettings.h"
 #include "themedmenu.h"
 #include "util.h"
 #include "remotefile.h"
-#include "lcddevice.h"
 #include "dialogbox.h"
 #include "mythdialogs.h"
 #include "mythplugin.h"
 #include "screensaver.h"
 #include "DisplayRes.h"
+#include "dbsettings.h"
+#include "langsettings.h"
+
+// These defines provide portability for different
+// plugin file names.
+#ifdef CONFIG_DARWIN
+const QString kPluginLibPrefix = "lib";
+const QString kPluginLibSuffix = ".dylib";
+#else
+const QString kPluginLibPrefix = "lib";
+const QString kPluginLibSuffix = ".so";
+#endif
 
 MythContext *gContext = NULL;
 
@@ -37,9 +50,24 @@ class MythContextPrivate
     MythContextPrivate(MythContext *lparent);
    ~MythContextPrivate();
 
-    void Init(bool gui, bool lcd);
+    void Init(bool gui);
+    bool IsWideMode() const {return (m_baseWidth == 1280);}
+    void SetWideMode() {m_baseWidth = 1280; m_baseHeight = 720;}
+    bool IsSquareMode() const {return (m_baseWidth == 800);}
+    void SetSquareMode() {m_baseWidth = 800; m_baseHeight = 600;}
+
 
     void LoadLogSettings(void);
+    void LoadDatabaseSettings(bool reload);
+    
+    bool FixSettingsFile(void);
+    bool WriteSettingsFile(const DatabaseParams &params,
+                           bool overwrite = false);
+    bool FindSettingsProbs(void);
+    bool RetryDatabaseConnection(QSqlDatabase *db);
+    QString getResponse(const QString &query, const QString &def);
+    int intResponse(const QString &query, int def);
+    bool PromptForDatabaseParams(void);
 
     MythContext *parent;
 
@@ -48,6 +76,8 @@ class MythContextPrivate
 
     QString m_installprefix;
 
+    bool m_gui;
+    bool m_backend;
     bool m_themeloaded;
     QString m_menuthemepathname;
     QString m_themepathname;
@@ -56,7 +86,8 @@ class MythContextPrivate
 
     int m_xbase, m_ybase;
     int m_height, m_width;
-
+    int m_baseWidth, m_baseHeight;
+    
     QString m_localhostname;
 
     QMutex serverSockLock;
@@ -68,8 +99,6 @@ class MythContextPrivate
 
     QMap<QString, QImage> imageCache;
 
-    LCD *lcd_device;
-
     QString language;
 
     MythMainWindow *mainWindow;
@@ -100,8 +129,11 @@ class MythContextPrivate
 
     QMutex *m_priv_mutex;
     queue<MythPrivRequest> m_priv_requests;
+    QWaitCondition m_priv_queued;
 };
 
+
+
 MythContextPrivate::MythContextPrivate(MythContext *lparent)
 {
     pluginmanager = NULL;
@@ -114,6 +146,25 @@ MythContextPrivate::MythContextPrivate(MythContext *lparent)
     else
         m_installprefix = PREFIX;
 
+    if (QDir(m_installprefix).isRelative())
+    {
+        // If the PREFIX is relative, evaluate it relative to our
+        // executable directory. This can be fragile on Unix, so
+        // use relative PREFIX values with care.
+#if QT_VERSION >= 0x030200
+        QDir prefixDir = qApp->applicationDirPath();
+#else
+        QString appPath = QFileInfo(qApp->argv()[0]).absFilePath();
+        QDir prefixDir(appPath.left(appPath.findRev("/")));
+#endif
+        prefixDir.cd(m_installprefix);
+        m_installprefix = prefixDir.canonicalPath();
+
+        VERBOSE(VB_ALL, QString("Using runtime prefix = %1")
+                                .arg(m_installprefix));
+    }
+
+    m_backend = false;
     m_settings = new Settings;
     m_qtThemeSettings = new Settings;
 
@@ -122,26 +173,15 @@ MythContextPrivate::MythContextPrivate(MythContext *lparent)
     m_backgroundimage = NULL;
 
     m_db = QSqlDatabase::addDatabase("QMYSQL3", "MythContext");
-    screensaver = new ScreenSaverControl();
+    screensaver = ScreenSaverControl::get();
+    m_baseHeight = 600;
+    m_baseWidth = 800;
 }
 
-void MythContextPrivate::Init(bool gui, bool lcd)
+void MythContextPrivate::Init(bool gui)
 {
-    if (!parent->LoadSettingsFiles("mysql.txt"))
-        cerr << "Unable to read configuration file mysql.txt" << endl;
-
-    m_localhostname = m_settings->GetSetting("LocalHostName", NULL);
-    if (m_localhostname == NULL)
-    {
-        char localhostname[1024];
-        if (gethostname(localhostname, 1024))
-        {
-            perror("gethostname");
-            VERBOSE(VB_IMPORTANT, "MCP: Could not determine host name, exiting");
-            exit(-29);
-        }
-        m_localhostname = localhostname;
-    }
+    m_gui = gui;
+    LoadDatabaseSettings(false);
 
     m_xbase = m_ybase = 0;
 
@@ -172,11 +212,6 @@ void MythContextPrivate::Init(bool gui, bool lcd)
 
     mainWindow = NULL;
 
-    if (lcd)
-        lcd_device = new LCD();
-    else
-        lcd_device = NULL;
-
     disablelibrarypopup = false;
 
     display_res = NULL;
@@ -196,8 +231,6 @@ MythContextPrivate::~MythContextPrivate()
         delete serverSock;
     if (eventSock)
         delete eventSock;
-    if (lcd_device)
-        delete lcd_device;
     if (m_priv_mutex)
         delete m_priv_mutex;
 }
@@ -209,7 +242,319 @@ void MythContextPrivate::LoadLogSettings(void)
     m_logprintlevel = parent->GetNumSetting("LogPrintLevel", LP_ERROR);
 }
 
-MythContext::MythContext(const QString &binversion, bool gui, bool lcd)
+void MythContextPrivate::LoadDatabaseSettings(bool reload)
+{
+    if (reload)
+    {
+        if (m_settings)
+            delete m_settings;
+        m_settings = new Settings;
+    }
+    
+    if (!parent->LoadSettingsFiles("mysql.txt"))
+    {
+        VERBOSE(VB_IMPORTANT, "Unable to read configuration file mysql.txt");
+        if (!FixSettingsFile())
+            exit(-29);
+        else
+            parent->LoadSettingsFiles("mysql.txt");
+    }
+
+    // Even if we have loaded the settings file, it may be incomplete,
+    // so we check for missing values and warn user
+    FindSettingsProbs();
+
+    m_localhostname = m_settings->GetSetting("LocalHostName", NULL);
+    if (m_localhostname == NULL)
+    {
+        char localhostname[1024];
+        if (gethostname(localhostname, 1024))
+        {
+            perror("gethostname");
+            VERBOSE(VB_IMPORTANT, "MCP: Could not determine host name, exiting");
+            exit(-29);
+        }
+        m_localhostname = localhostname;
+    }
+}
+
+bool MythContextPrivate::FixSettingsFile(void)
+{
+    DatabaseParams defaultParams = parent->GetDatabaseParams();
+    
+    return WriteSettingsFile(defaultParams);
+}
+
+bool MythContextPrivate::WriteSettingsFile(const DatabaseParams &params,
+                                           bool overwrite)
+{
+    QString path = QDir::homeDirPath() + "/.mythtv/mysql.txt";
+    QFile   * f  = new QFile(path);
+    
+    if (!overwrite && f->exists())
+    {
+        return false;
+    }
+
+    QString dirpath = QDir::homeDirPath() + "/.mythtv";
+    QDir createDir(dirpath);
+
+    if (!createDir.exists())
+    {
+        if (!createDir.mkdir(dirpath, true))
+        {
+            VERBOSE(VB_IMPORTANT, "Could not create ~/.mythtv");
+            return false;
+        }
+    }
+
+    if (!f->open(IO_WriteOnly))
+    {
+        VERBOSE(VB_IMPORTANT, QString("Could not open settings file %1 "
+                                      "for writing").arg(path));
+        return false;
+    }
+    
+    VERBOSE(VB_IMPORTANT, QString("Writing settings file %1").arg(path));
+    QTextStream s(f);
+    s << "DBHostName=" << params.dbHostName << endl
+      << "DBUserName=" << params.dbUserName << endl
+      << "DBPassword=" << params.dbPassword << endl
+      << "DBName="     << params.dbName     << endl
+      << "DBType="     << params.dbType     << endl
+      << endl
+      << "# Set the following if you want to use something other than the\n"
+      << "# machine's real hostname for identifying settings in the database.\n"
+      << "# This is useful if your hostname changes often, as otherwise\n"
+      << "# you'll need to reconfigure mythtv (or futz with the DB) every time.\n"
+      << "# TWO HOSTS MUST NOT USE THE SAME VALUE\n"
+      << "#\n";
+    
+    if (params.localEnabled)
+        s << "LocalHostName=" << params.localHostName << endl;
+    else
+        s << "#LocalHostName=my-unique-identifier-goes-here\n";
+    
+    s << endl
+      << "# If you want your frontend to be able to wake your MySQL server\n"
+      << "# using WakeOnLan, have a look at the following settings:\n"
+      << "#\n"
+      << "# Set the time the frontend waits (in seconds) between reconnect tries.\n"
+      << "# This should be the rough time your MySQL server needs for startup\n";
+
+    if (params.wolEnabled)
+        s << "WOLsqlReconnectWaitTime=" << params.wolReconnect << endl;
+    else
+        s << "#WOLsqlReconnectWaitTime=0\n";
+    
+    s << "#\n"
+      << "#\n"
+      << "# This is the amount of retries to wake the MySQL server until the frontend\n"
+      << "# gives up\n";
+     
+    if (params.wolEnabled)
+        s << "WOLsqlConnectRetry=" << params.wolRetry << endl;
+    else
+        s << "#WOLsqlConnectRetry=5\n";
+    
+    s << "#\n"
+      << "#\n"
+      << "# This is the command executed to wake your MySQL server.\n";
+    
+    if (params.wolEnabled)
+        s << "WOLsqlCommand=" << params.wolCommand << endl;
+    else
+        s << "#WOLsqlCommand=echo 'WOLsqlServerCommand not set'\n";
+    
+    f->close();
+    return true;
+}
+
+bool MythContextPrivate::FindSettingsProbs(void)
+{
+    bool problems = false;
+    
+    if (m_settings->GetSetting("DBHostName").isEmpty())
+    {
+        problems = true;
+        VERBOSE(VB_IMPORTANT, "DBHostName is not set in mysql.txt");
+    }
+    if (m_settings->GetSetting("DBUserName").isEmpty())
+    {
+        problems = true;
+        VERBOSE(VB_IMPORTANT, "DBUserName is not set in mysql.txt");
+    }
+    if (m_settings->GetSetting("DBPassword").isEmpty())
+    {
+        problems = true;
+        VERBOSE(VB_IMPORTANT, "DBPassword is not set in mysql.txt");
+    }
+    if (m_settings->GetSetting("DBName").isEmpty())
+    {
+        problems = true;
+        VERBOSE(VB_IMPORTANT, "DBName is not set in mysql.txt");
+    }
+    return problems;
+}
+
+bool MythContextPrivate::RetryDatabaseConnection(QSqlDatabase *db)
+{
+    while (!parent->OpenDatabase(db, false))
+    {
+        printf("couldn't open db\n");
+        int pause = m_settings->GetNumSetting("WOLsqlReconnectWaitTime", 0);
+        if (pause > 0)
+        {
+            int acttry = 1;
+            int retries = m_settings->GetNumSetting("WOLsqlConnectRetry", 5);
+
+            QString WOLsqlCommand = m_settings->GetSetting("WOLsqlCommand", 
+                                                   "echo \'WOLsqlServerCommand "
+                                                   "not set\nPlease do so in "
+                                                   "your mysql.txt!\'");
+
+            if (!WOLsqlCommand.isEmpty())
+            {
+                while (acttry <= retries && !parent->OpenDatabase(db, false))
+                {
+                    printf("Trying to wakeup SQLserver (Try %d of %d)\n",
+                           acttry, retries);
+                    system(WOLsqlCommand);
+                    sleep(pause);
+                    ++acttry;
+                }
+            }
+
+            if (WOLsqlCommand.isEmpty() || (acttry > retries))
+            {
+                printf("Sorry, couldn't open db\n");
+                if (!PromptForDatabaseParams())
+                    return false;
+            }
+        }
+        else if (!PromptForDatabaseParams())
+            return false;
+    }
+    return true;
+}
+
+QString MythContextPrivate::getResponse(const QString &query,
+                                        const QString &def)
+{
+    cout << query;
+
+    if (def != "")
+        cout << " [" << def << "]  ";
+    else
+        cout << "  ";
+    
+    char response[80];
+    cin.clear();
+    cin.getline(response, 80);
+    if (!cin.good())
+    {
+        cout << endl;
+        VERBOSE(VB_IMPORTANT, "Read from stdin failed");
+        return NULL;
+    }
+
+    QString qresponse = response;
+
+    if (qresponse == "")
+        qresponse = def;
+
+    return qresponse;
+}
+
+int MythContextPrivate::intResponse(const QString &query, int def)
+{
+    QString str_resp = getResponse(query, QString("%1").arg(def));
+    if (!str_resp)
+        return false;
+    bool ok;
+    int resp = str_resp.toInt(&ok);
+    return (ok ? resp : def);
+}
+
+bool MythContextPrivate::PromptForDatabaseParams(void)
+{
+    bool accepted = false;
+    if (m_gui)
+    {
+        // construct minimal MythMainWindow
+        m_settings->SetSetting("Theme", "blue");
+#ifdef Q_WS_MACX
+        // Myth looks horrible in default Mac style for Qt
+        m_settings->SetSetting("Style", "Windows");
+#endif
+        parent->LoadQtConfig();
+        MythMainWindow *mainWindow = new MythMainWindow();
+        parent->SetMainWindow(mainWindow);
+    
+        // ask user for language settings
+        LanguageSettings::prompt();
+        
+        // ask user for database parameters
+        DatabaseSettings settings;
+        accepted = (settings.exec(NULL) == QDialog::Accepted);
+        if (!accepted)
+            VERBOSE(VB_IMPORTANT, "User canceled database configuration");
+    
+        // tear down temporary main window
+        delete mainWindow;
+    }
+    else
+    {
+        DatabaseParams params = parent->GetDatabaseParams();
+        QString response;
+        
+        // give user chance to skip config
+        response = getResponse("Would you like to configure the database "
+                               "connection now?",
+                               "yes");
+        if (!response || response.left(1).lower() != "y")
+            return false;
+        
+        params.dbHostName = getResponse("Database host name:",
+                                        params.dbHostName);
+        params.dbName     = getResponse("Database name:",
+                                        params.dbName);
+        params.dbUserName = getResponse("Database user name:",
+                                        params.dbUserName);
+        params.dbPassword = getResponse("Database password:",
+                                        params.dbPassword);
+        
+        params.localHostName = getResponse("Unique identifier for this machine "
+                                           "(if empty, the local host name "
+                                           "will be used):",
+                                           params.localHostName);
+        params.localEnabled = !params.localHostName.isEmpty();
+        
+        response = getResponse("Would you like to use Wake-On-LAN to retry "
+                               "database connections?",
+                               (params.wolEnabled ? "no" : "yes"));
+        if (response.left(1).lower() == "y")
+        {
+            params.wolEnabled   = true;
+            params.wolReconnect = intResponse("Seconds to wait for "
+                                              "reconnection:",
+                                              params.wolReconnect);
+            params.wolRetry     = intResponse("Number of times to retry:",
+                                              params.wolRetry);
+            params.wolCommand   = getResponse("Command to use to wake server:",
+                                              params.wolCommand);
+        }
+        else
+            params.wolEnabled   = false;
+        
+        accepted = parent->SaveDatabaseParams(params);
+    }
+    if (accepted)
+        LoadDatabaseSettings(true);
+    return accepted;
+}
+
+MythContext::MythContext(const QString &binversion, bool gui)
            : QObject()
 {
     qInitNetworkProtocols();
@@ -226,7 +571,7 @@ MythContext::MythContext(const QString &binversion, bool gui, bool lcd)
 
     d = new MythContextPrivate(this);
 
-    d->Init(gui, lcd);
+    d->Init(gui);
 
     connect(d->eventSock, SIGNAL(connected()), 
             this, SLOT(EventSocketConnected()));
@@ -322,13 +667,15 @@ bool MythContext::ConnectServer(const QString &hostname, int port)
         return false;
     }
 
+    // called with the lock
     QString str = QString("ANN Playback %1 %2")
                          .arg(d->m_localhostname).arg(false);
     QStringList strlist = str;
     WriteStringList(d->serverSock, strlist);
     ReadStringList(d->serverSock, strlist, true);
-    
-    d->eventSock->connectToHost(hostname, port);
+
+    if (d->eventSock->state() == QSocket::Idle)    
+        d->eventSock->connectToHost(hostname, port);
     
     return true;
 }
@@ -338,12 +685,26 @@ bool MythContext::IsConnectedToMaster(void)
     return d->serverSock;
 }
 
+void MythContext::SetBackend(bool backend)
+{
+    d->m_backend = backend;
+}
+
+bool MythContext::IsBackend(void)
+{
+    return d->m_backend;
+}
+
 QString MythContext::GetMasterHostPrefix(void)
 {
     QString ret = "";
 
     if (!d->serverSock)
+    {
+        d->serverSockLock.lock();
         ConnectToMasterServer();
+        d->serverSockLock.unlock();
+    }
     
     if (d->serverSock)
         ret = QString("myth://%1:%2/")
@@ -372,6 +733,66 @@ QString MythContext::GetShareDir(void)
     return d->m_installprefix + "/share/mythtv/"; 
 }
 
+QString MythContext::GetLibraryDir(void) 
+{ 
+    return d->m_installprefix + "/lib/mythtv/"; 
+}
+
+QString MythContext::GetThemesParentDir(void) 
+{ 
+    return GetShareDir() + "themes/"; 
+}
+
+QString MythContext::GetPluginsDir(void) 
+{ 
+    return GetLibraryDir() + "plugins/"; 
+}
+
+QString MythContext::GetPluginsNameFilter(void) 
+{ 
+    return kPluginLibPrefix + "*" + kPluginLibSuffix; 
+}
+
+QString MythContext::FindPlugin(const QString &plugname) 
+{ 
+    return GetPluginsDir() + kPluginLibPrefix + plugname + kPluginLibSuffix; 
+}
+
+QString MythContext::GetTranslationsDir(void) 
+{ 
+    return GetShareDir() + "i18n/"; 
+}
+
+QString MythContext::GetTranslationsNameFilter(void) 
+{ 
+    return "mythfrontend_*.qm"; 
+}
+
+QString MythContext::FindTranslation(const QString &translation) 
+{ 
+    return GetTranslationsDir() + "mythfrontend_" + translation.lower() + ".qm"; 
+}
+
+QString MythContext::GetFontsDir(void) 
+{ 
+    return GetShareDir();
+}
+
+QString MythContext::GetFontsNameFilter(void) 
+{ 
+    return "*ttf"; 
+}
+
+QString MythContext::FindFont(const QString &fontname) 
+{ 
+    return GetFontsDir() + fontname + ".ttf"; 
+}
+
+QString MythContext::GetFiltersDir(void) 
+{ 
+    return GetLibraryDir() + "filters/"; 
+}
+
 bool MythContext::LoadSettingsFiles(const QString &filename)
 {
     return d->m_settings->LoadSettingsFiles(filename, d->m_installprefix);
@@ -414,9 +835,21 @@ void MythContext::LoadQtConfig(void)
     if (style != "")
         qApp->setStyle(style);
 
-    QString themename = GetSetting("Theme");
+    QString themename = GetSetting("Theme");    
     QString themedir = FindThemeDir(themename);
     
+    if (themename.contains("-wide", false))
+    {
+        VERBOSE( VB_ALL, QString("Switching to wide mode (%1)").arg(themename));
+        d->SetWideMode();
+    }
+    else
+    {
+        VERBOSE( VB_ALL, QString("Switching to square mode (%1)").arg(themename));
+        d->SetSquareMode();
+    }
+
+        
     d->m_themepathname = themedir + "/";
     
     themedir += "/qtlook.txt";
@@ -534,7 +967,7 @@ void MythContext::CacheThemeImages(void)
 {
     QString baseDir = d->m_installprefix + "/share/mythtv/themes/default/";
 
-    if (d->m_screenwidth == 800 && d->m_screenheight == 600)
+    if (d->m_screenwidth == d->m_baseWidth && d->m_screenheight == d->m_baseHeight)
         return;
 
     CacheThemeImagesDirectory(d->m_themepathname);
@@ -706,8 +1139,8 @@ void MythContext::InitializeScreenSettings()
         height = 480;
     }
 
-    d->m_wmult = width / 800.0;
-    d->m_hmult = height / 600.0;
+    d->m_wmult = width / (float)d->m_baseWidth;
+    d->m_hmult = height / (float)d->m_baseHeight;
     d->m_screenwidth = width;   
     d->m_screenheight = height;
 
@@ -757,27 +1190,46 @@ QString MythContext::GetThemeDir(void)
     return d->m_themepathname;
 }
 
-int MythContext::OpenDatabase(QSqlDatabase *db)
+int MythContext::OpenDatabase(QSqlDatabase *db, bool promptOnFailure)
 {
+    if (promptOnFailure)
+    {
+        // RetryDatabaseConnection will use GUI or console prompts
+        // to edit settings if connection fails
+        return (int)(d->RetryDatabaseConnection(db));
+    }
+    
+    int res = 1;
+    
     d->dbLock.lock();
     if (!d->m_db->isOpen()) {
         d->m_db->setDatabaseName(d->m_settings->GetSetting("DBName"));
         d->m_db->setUserName(d->m_settings->GetSetting("DBUserName"));
         d->m_db->setPassword(d->m_settings->GetSetting("DBPassword"));
         d->m_db->setHostName(d->m_settings->GetSetting("DBHostName"));
-        d->m_db->open();
+        res = d->m_db->open();
     }
     d->dbLock.unlock();
-        
-    db->setDatabaseName(d->m_settings->GetSetting("DBName"));
-    db->setUserName(d->m_settings->GetSetting("DBUserName"));
-    db->setPassword(d->m_settings->GetSetting("DBPassword"));
-    db->setHostName(d->m_settings->GetSetting("DBHostName"));
- 
-    int res = db->open();
+    
     if (!res)
-        cerr << "Unable to connect to database!" << endl
-             << DBErrorMessage(db->lastError()) << endl;
+    {
+        VERBOSE(VB_IMPORTANT, "Unable to connect to database!");
+        VERBOSE(VB_IMPORTANT, DBErrorMessage(d->m_db->lastError()));
+    }
+    else if (db)
+    {
+        db->setDatabaseName(d->m_settings->GetSetting("DBName"));
+        db->setUserName(d->m_settings->GetSetting("DBUserName"));
+        db->setPassword(d->m_settings->GetSetting("DBPassword"));
+        db->setHostName(d->m_settings->GetSetting("DBHostName"));
+        
+        res = db->open();
+        if (!res)
+        {
+            VERBOSE(VB_IMPORTANT, "Unable to connect to database!");
+            VERBOSE(VB_IMPORTANT, DBErrorMessage(db->lastError()));
+        }
+    }
     return res;
 }
 
@@ -812,9 +1264,16 @@ void MythContext::DBError(const QString &where, const QSqlQuery& query)
         cerr << "DB Error (" << where << "):" << endl;
     }
 
+#if QT_VERSION >= 0x030200
+    cerr << "Query was:" << endl
+         << query.executedQuery() << endl
+         << DBErrorMessage(query.lastError()) << endl;
+#else
+    cerr << "Your version of Qt is too old to provide proper debugging\n";
     cerr << "Query was:" << endl
          << query.lastQuery() << endl
          << DBErrorMessage(query.lastError()) << endl;
+#endif
 }
 
 QString MythContext::DBErrorMessage(const QSqlError& err)
@@ -1077,7 +1536,6 @@ QImage *MythContext::LoadScaleImage(QString filename, bool fromcache)
 {
     if (filename.left(5) == "myth:")
         return NULL;
-
     QString baseDir = d->m_installprefix + "/share/mythtv/themes/default/";
 
     QFile checkFile(filename);
@@ -1111,13 +1569,24 @@ QImage *MythContext::LoadScaleImage(QString filename, bool fromcache)
 
     if (!checkFile.exists())
     {
-        QFileInfo fi(filename);
-        filename = d->m_themepathname + fi.fileName();
-        checkFile.setName(filename);
+        QString fileTemp;
+        fileTemp = d->m_themepathname + filename;
+        checkFile.setName(fileTemp);
         if (!checkFile.exists())
-            filename = baseDir + fi.fileName();
+        {
+            QFileInfo fi(filename);
+            filename = d->m_themepathname + fi.fileName();
+            checkFile.setName(filename);
+            if (!checkFile.exists())
+                filename = baseDir + fi.fileName();
+        }
+        else
+        {
+            filename = fileTemp;
+        }
     }
 
+
     QImage *ret = NULL;
 
     int width, height;
@@ -1125,7 +1594,7 @@ QImage *MythContext::LoadScaleImage(QString filename, bool fromcache)
 
     GetScreenSettings(width, wmult, height, hmult);
 
-    if (width != 800 || height != 600)
+    if (width != d->m_baseWidth || height != d->m_baseHeight)
     {
         QImage tmpimage;
 
@@ -1156,9 +1625,7 @@ QPixmap *MythContext::LoadScalePixmap(QString filename, bool fromcache)
 {
     if (filename.left(5) == "myth:")
         return NULL;
-
     QString baseDir = d->m_installprefix + "/share/mythtv/themes/default/";
-
     QFile checkFile(filename);
     QFileInfo fi(filename);
 
@@ -1190,11 +1657,22 @@ QPixmap *MythContext::LoadScalePixmap(QString filename, bool fromcache)
 
     if (!checkFile.exists())
     {
-        QFileInfo fi(filename);
-        filename = d->m_themepathname + fi.fileName();
-        checkFile.setName(filename);
+        QString fileTemp;
+        fileTemp = d->m_themepathname + filename;
+        checkFile.setName(fileTemp);
         if (!checkFile.exists())
-            filename = baseDir + fi.fileName();
+        {
+            QFileInfo fi(filename);
+            filename = d->m_themepathname + fi.fileName();
+            checkFile.setName(filename);
+            if (!checkFile.exists())
+                filename = baseDir + fi.fileName();
+        }
+        else
+        {
+            filename = fileTemp;
+        }
+
     }
         
     QPixmap *ret = new QPixmap();
@@ -1204,7 +1682,7 @@ QPixmap *MythContext::LoadScalePixmap(QString filename, bool fromcache)
 
     GetScreenSettings(width, wmult, height, hmult);
 
-    if (width != 800 || height != 600)
+    if (width != d->m_baseWidth || height != d->m_baseHeight)
     {           
         QImage tmpimage;
 
@@ -1267,8 +1745,6 @@ void MythContext::SetSetting(const QString &key, const QString &newValue)
     d->m_settings->SetSetting(key, newValue);
 }
 
-
-
 bool MythContext::SendReceiveStringList(QStringList &strlist, bool quickTimeout)
 {
     d->serverSockLock.lock();
@@ -1330,7 +1806,10 @@ void MythContext::EventSocketRead(void)
         QString prefix = strlist[0];
         QString message = strlist[1];
         
-        if (prefix != "BACKEND_MESSAGE")
+        if (prefix == "OK")
+        {
+        }
+        else if (prefix != "BACKEND_MESSAGE")
         {
             cerr << "Received a: " << prefix << " message from the backend\n";
             cerr << "But I don't know what to do with it.\n";
@@ -1395,14 +1874,17 @@ bool MythContext::CheckProtoVersion(QSocket* socket)
 
 void MythContext::EventSocketConnected(void)
 {
+/* Unnecessary, just checked on the serverSock connect */
+#if 0
     if (!CheckProtoVersion(d->eventSock))
         return;
+#endif
 
     QString str = QString("ANN Playback %1 %2")
                          .arg(d->m_localhostname).arg(true);
     QStringList strlist = str;
     WriteStringList(d->eventSock, strlist);
-    ReadStringList(d->eventSock, strlist);
+//    ReadStringList(d->eventSock, strlist);
 }
 
 void MythContext::EventSocketClosed(void)
@@ -1444,18 +1926,18 @@ void MythContext::dispatchNow(MythEvent &e)
 
 QFont MythContext::GetBigFont(void)
 {
-    return QFont("Arial", (int)ceil(d->bigfontsize * d->m_hmult), QFont::Bold);
+    return QFont("Arial", (int)floor(d->bigfontsize * d->m_hmult), QFont::Bold);
 }
 
 QFont MythContext::GetMediumFont(void)
 {
-    return QFont("Arial", (int)ceil(d->mediumfontsize * d->m_hmult), 
+    return QFont("Arial", (int)floor(d->mediumfontsize * d->m_hmult), 
                  QFont::Bold);
 }
 
 QFont MythContext::GetSmallFont(void)
 {
-    return QFont("Arial", (int)ceil(d->smallfontsize * d->m_hmult), 
+    return QFont("Arial", (int)floor(d->smallfontsize * d->m_hmult), 
                  QFont::Bold);
 }
 
@@ -1477,11 +1959,6 @@ MythMainWindow *MythContext::GetMainWindow(void)
     return d->mainWindow;
 }
 
-LCD *MythContext::GetLCDDevice(void)
-{
-    return d->lcd_device;
-}
-
 bool MythContext::TestPopupVersion(const QString &name, 
                                    const QString &libversion,
                                    const QString &pluginversion)
@@ -1600,22 +2077,26 @@ void MythContext::LogEntry(const QString &module, int priority,
         if (d->m_db->isOpen())
         {
             KickDatabase(d->m_db);
-    
-            QString querystr = QString("INSERT INTO mythlog (module, priority, "
-                                       "logdate, host, message, details) "
-                                       "values ( '%1', %2, now(), '%3', "
-                                       "'%4','%5' );") 
-                                       .arg(module) 
-                                       .arg(priority)
-                                       .arg(d->m_localhostname)
-                                       .arg(message).arg(details);
-    
-            QSqlQuery result = d->m_db->exec(querystr);
-            if (!result.isActive())
-                MythContext::DBError("LogEntry", querystr);
+
+            QSqlQuery result(QString::null, d->m_db);
+
+            result.prepare("INSERT INTO mythlog (module, priority, "
+                           "logdate, host, message, details) "
+                           "values (:MODULE, :PRIORITY, now(), :HOSTNAME, "
+                           ":MESSAGE, :DETAILS );");
+
+            result.bindValue(":MODULE", module);
+            result.bindValue(":PRIORITY", priority);
+            result.bindValue(":HOSTNAME", d->m_localhostname);
+            result.bindValue(":MESSAGE", message);
+            result.bindValue(":DETAILS", details);
+
+            if (!result.exec() || !result.isActive())
+                MythContext::DBError("LogEntry", result);
 
             if (d->m_logmaxcount > 0)
             {
+                QString querystr;
                 querystr = QString("SELECT logid FROM mythlog WHERE "
                                    "module='%1' ORDER BY logdate ASC") 
                                    .arg(module);
@@ -1647,12 +2128,17 @@ void MythContext::addPrivRequest(MythPrivRequest::Type t, void *data)
 {
     QMutexLocker lockit(d->m_priv_mutex);
     d->m_priv_requests.push(MythPrivRequest(t, data));
+    d->m_priv_queued.wakeAll();
 }
 
-bool MythContext::hasPrivRequest() const
+void MythContext::waitPrivRequest() const
 {
-    QMutexLocker lockit(d->m_priv_mutex);
-    return !d->m_priv_requests.empty();
+    while (true) 
+    {
+        d->m_priv_queued.wait();
+        if (!d->m_priv_requests.empty())
+            return;
+    }
 }
 
 MythPrivRequest MythContext::popPrivRequest()
@@ -1665,3 +2151,59 @@ MythPrivRequest MythContext::popPrivRequest()
     }
     return ret_val;
 }
+
+DatabaseParams MythContext::GetDatabaseParams(void)
+{
+    DatabaseParams params;
+    
+    params.dbHostName = d->m_settings->GetSetting("DBHostName", "localhost");
+    params.dbUserName = d->m_settings->GetSetting("DBUserName", "mythtv");
+    params.dbPassword = d->m_settings->GetSetting("DBPassword", "mythtv");
+    params.dbName     = d->m_settings->GetSetting("DBName",     "mythconverg");
+    params.dbType     = d->m_settings->GetSetting("DBType",     "QMYSQL3");
+    
+    params.localHostName = d->m_settings->GetSetting("LocalHostName", "");
+    if (params.localHostName.isEmpty())
+    {
+        params.localEnabled = false;
+        params.localHostName = "my-unique-identifier-goes-here";
+    }
+    else
+        params.localEnabled = true;
+    
+    params.wolReconnect = d->m_settings->GetNumSetting("WOLsqlReconnectWaitTime", -1);
+    if (params.wolReconnect == -1)
+    {
+        params.wolEnabled = false;
+        params.wolReconnect = 0;
+    }
+    else
+        params.wolEnabled = true;
+
+    params.wolRetry = d->m_settings->GetNumSetting("WOLsqlConnectRetry", 5);
+    params.wolCommand = d->m_settings->GetSetting("WOLsqlCommand", "echo 'WOLsqlServerCommand not set'");
+    
+    return params;
+}
+
+bool MythContext::SaveDatabaseParams(const DatabaseParams &params)
+{
+    DatabaseParams cur_params = GetDatabaseParams();
+    
+    // only rewrite file if it hasn't changed
+    if (params.dbHostName   != cur_params.dbHostName          ||
+        params.dbUserName   != cur_params.dbUserName          ||
+        params.dbPassword   != cur_params.dbPassword          ||
+        params.dbName       != cur_params.dbName              ||
+        params.dbType       != cur_params.dbType              ||
+        params.localEnabled != cur_params.localEnabled        ||
+        params.wolEnabled   != cur_params.wolEnabled          ||
+        (params.localEnabled &&
+         (params.localHostName != cur_params.localHostName))  ||
+        (params.wolEnabled &&
+         (params.wolReconnect  != cur_params.wolReconnect ||
+          params.wolRetry      != cur_params.wolRetry     ||
+          params.wolCommand    != cur_params.wolCommand)))
+        return d->WriteSettingsFile(params, true);
+    return true;
+}
