@@ -115,6 +115,7 @@ int RTjpeg::b2s(int16_t *data, int8_t *strm, uint8_t bt8)
  register unsigned char bitten;
  register unsigned char bitoff;
 
+ uint8_t *ustrm = (uint8_t *)strm;
  bt8 = bt8;
 #ifdef SHOWBLOCK
 
@@ -132,7 +133,7 @@ int RTjpeg::b2s(int16_t *data, int8_t *strm, uint8_t bt8)
 // return 2;
 
  // first byte allways written
- (uint8_t)strm[0]=
+ ustrm[0]=
       (uint8_t)(data[RTjpeg_ZZ[0]]>254) ? 254:((data[RTjpeg_ZZ[0]]<0)?0:data[RTjpeg_ZZ[0]]);
 
 
@@ -142,7 +143,7 @@ int RTjpeg::b2s(int16_t *data, int8_t *strm, uint8_t bt8)
  bitten = ((unsigned char)ci) << 2;
 
  if (ci==0) {
-   (uint8_t)strm[1]= bitten; 
+   ustrm[1]= bitten; 
    co = 2;
    return (int)co;
  }
@@ -171,7 +172,7 @@ int RTjpeg::b2s(int16_t *data, int8_t *strm, uint8_t bt8)
    }
 
    if( bitoff == 0 ) {
-      (uint8_t)strm[co]= bitten; 
+      ustrm[co]= bitten; 
       bitten = 0;
       bitoff = 8; 
       co++;
@@ -183,7 +184,7 @@ int RTjpeg::b2s(int16_t *data, int8_t *strm, uint8_t bt8)
  /* ci must be 0 */
  if(bitoff != 6) {
 
-      (uint8_t)strm[co]= bitten; 
+      ustrm[co]= bitten; 
       co++;
      
  }
@@ -200,7 +201,7 @@ HERZWEH:
    break;
  case 2:
  case 0:
-   (uint8_t)strm[co]= bitten; 
+   ustrm[co]= bitten; 
    bitoff = 4;
    co++;
    bitten = 0; // clear half nibble values in bitten
@@ -221,7 +222,7 @@ HERZWEH:
    bitten |= (ZZvalue&0xf)<<bitoff;
 
    if( bitoff == 0 ) {
-      (uint8_t)strm[co]= bitten; 
+      ustrm[co]= bitten; 
       bitten = 0;
       bitoff = 8;
       co++;
@@ -231,14 +232,14 @@ HERZWEH:
  
  /* ci must be 0 */
  if( bitoff == 0 ) {
-    (uint8_t)strm[co]= bitten; 
+    ustrm[co]= bitten; 
     co++;
  }  
  goto BAUCHWEH;
 
 HIRNWEH:
 
- (uint8_t)strm[co]= bitten; 
+ ustrm[co]= bitten; 
  co++;
   
 
