@@ -4,6 +4,7 @@
 
 #include <qfile.h>
 #include <qstring.h>
+#include <cerrno>
 
 // XXX Program duration should be stored as seconds, not as a QTime.
 //     limited to 24 hours this way.
@@ -51,10 +52,14 @@ bool DDStructureParser::startElement(const QString &pnamespaceuri,
     }
     else if (currtagname == "map") 
     {
+        int tmpindex;
         curr_lineupmap.clearValues();
         curr_lineupmap.lineupid = curr_lineup.lineupid;
         curr_lineupmap.stationid = pxmlatts.value("station");
         curr_lineupmap.channel = pxmlatts.value("channel");
+        tmpindex = pxmlatts.index("channelMinor"); // for ATSC
+        if (tmpindex != -1) 
+            curr_lineupmap.channelMinor = pxmlatts.value(tmpindex);
     } 
     else if (currtagname == "schedule") 
     {
@@ -132,7 +137,7 @@ bool DDStructureParser::endElement(const QString &pnamespaceuri,
         query.bindValue(":CALLSIGN", curr_station.callsign);
         query.bindValue(":STATIONNAME", curr_station.stationname);
         query.bindValue(":AFFILIATE", curr_station.affiliate);
-        query.bindValue(" :FCCCHANNELNUMBER", curr_station.fccchannelnumber);
+        query.bindValue(":FCCCHANNELNUMBER", curr_station.fccchannelnumber);
 
         if (!query.exec())
             MythContext::DBError("Inserting into dd_station", query);
@@ -156,11 +161,13 @@ bool DDStructureParser::endElement(const QString &pnamespaceuri,
     {
         parent.lineupmaps.append(curr_lineupmap);
 
-        query.prepare("INSERT INTO dd_lineupmap (lineupid, stationid, channel) "
-                      "VALUES(:LINEUPID,:STATIONID,:CHANNEL);");
+        query.prepare("INSERT INTO dd_lineupmap (lineupid, stationid, "
+                      "channel, channelMinor) "
+                      "VALUES(:LINEUPID,:STATIONID,:CHANNEL,:CHANNELMINOR);");
         query.bindValue(":LINEUPID", curr_lineupmap.lineupid);
         query.bindValue(":STATIONID", curr_lineupmap.stationid);
         query.bindValue(":CHANNEL", curr_lineupmap.channel);
+        query.bindValue(":CHANNELMINOR", curr_lineupmap.channelMinor);
         if (!query.exec())
             MythContext::DBError("Inserting into dd_lineupmap", query);
     }
@@ -207,11 +214,12 @@ bool DDStructureParser::endElement(const QString &pnamespaceuri,
 
         if (prefix == "MV")
            cat_type = "movie";
-        else if (prefix == "EP")
-           cat_type = "series";
         else if (prefix == "SP")
            cat_type = "sports";
-        else if (prefix == "SH")
+        else if (prefix == "EP" ||
+                 curr_program.showtype.contains("series", false))
+           cat_type = "series";
+        else
            cat_type = "tvshow";
 
         query.prepare("INSERT INTO dd_program (programid, title, subtitle, "
@@ -298,9 +306,9 @@ bool DDStructureParser::characters(const QString& pchars)
     {
         if (pchars.contains("expire"))
         {
-	   QString ExtractDateFromMessage = pchars.right(20);
+           QString ExtractDateFromMessage = pchars.right(20);
            QDateTime EDFM = QDateTime::fromString(ExtractDateFromMessage, Qt::ISODate);
-	   QString ExpirationDate = EDFM.toString(Qt::LocalDate);
+           QString ExpirationDate = EDFM.toString(Qt::LocalDate);
            QString ExpirationDateMessage = "Your subscription expires on " + ExpirationDate;
            cout << ExpirationDateMessage << endl;
 
@@ -377,9 +385,10 @@ void DataDirectProcessor::updateStationViewTable()
         MythContext::DBError("Truncating temporary table dd_v_station", query);
 
     querystr = QString("INSERT INTO dd_v_station (stationid, callsign, "
-                       "stationname, affiliate, fccchannelnumber, channel) "
+                       "stationname, affiliate, fccchannelnumber, channel, "
+                       "channelMinor) "
                        "SELECT dd_station.stationid, callsign, stationname, "
-                       "affiliate, fccchannelnumber, channel "
+                       "affiliate, fccchannelnumber, channel, channelMinor "
                        "FROM dd_station, dd_lineupmap WHERE "
                        " ( (dd_station.stationid = dd_lineupmap.stationid) AND "
                        "   ( dd_lineupmap.lineupid = \"%0\") );")
@@ -425,70 +434,103 @@ void DataDirectProcessor::updateProgramViewTable(int sourceid)
         MythContext::DBError("Analyzing table dd_productioncrew", query);
 }
 
-void DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate, 
+void DataDirectProcessor::setInputFile(const QString &filename)
+{
+    inputfilename = filename;
+}
+
+FILE *DataDirectProcessor::getInputFile(bool plineupsOnly, QDateTime pstartDate,
+            QDateTime pendDate, QString &err_txt, QString &tmpfilename)
+{
+    FILE *ret = NULL;
+    if (inputfilename.isNull())
+    {
+        QString ddurl("http://datadirect.webservices.zap2it.com/tvlistings/xtvdService");
+
+        if (plineupsOnly) 
+        {
+            pstartDate = QDateTime(QDate::currentDate().addDays(2),
+                                   QTime::QTime(23, 59, 0));
+            pendDate = pstartDate;
+            pendDate = pendDate.addSecs(1);
+        }
+
+        QString startdatestr = pstartDate.toString(Qt::ISODate) + "Z";
+        QString enddatestr = pendDate.toString(Qt::ISODate) + "Z";
+
+        char ctempfilename[] = "/tmp/mythpostXXXXXX";
+        if (mkstemp(ctempfilename) == -1) 
+        {
+            VERBOSE(VB_IMPORTANT,
+                    QString("DDP: error creating temp files -- %1").
+                    arg(strerror(errno)));
+            return NULL;
+        }
+
+        tmpfilename = QString(ctempfilename);
+        QFile postfile(tmpfilename);
+
+        if (postfile.open(IO_WriteOnly)) 
+        {
+            QTextStream poststream(&postfile);
+            poststream << "<?xml version='1.0' encoding='utf-8'?>\n";
+            poststream << "<SOAP-ENV:Envelope\n";
+            poststream << "xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/'\n";
+            poststream << "xmlns:xsd='http://www.w3.org/2001/XMLSchema'\n";
+            poststream << "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'\n";
+            poststream << "xmlns:SOAP-ENC='http://schemas.xmlsoap.org/soap/encoding/'>\n";
+            poststream << "<SOAP-ENV:Body>\n";
+            poststream << "<ns1:download  xmlns:ns1='urn:TMSWebServices'>\n";
+            poststream << "<startTime xsi:type='xsd:dateTime'>";
+            poststream << startdatestr << "</startTime>\n";
+            poststream << "<endTime xsi:type='xsd:dateTime'>";
+            poststream << enddatestr << "</endTime>\n";
+            poststream << "</ns1:download>\n";
+            poststream << "</SOAP-ENV:Body>\n";
+            poststream << "</SOAP-ENV:Envelope>\n";
+        }
+        else
+        {
+            err_txt = "Unable to open post data output file.";
+            return NULL;
+        }
+
+        postfile.close();
+
+        QString command = QString("wget --http-user='%1' --http-passwd='%2' "
+                                  "--post-file='%3' --header='Accept-Encoding:gzip'"
+                                  " %4 --output-document=- | gzip -df")
+                                 .arg(getUserID())
+                                 .arg(getPassword())
+                                 .arg(tmpfilename)
+                                 .arg(ddurl);
+
+        err_txt = command;
+
+        ret = popen(command.ascii(), "r");
+    }
+    else
+    {
+        err_txt = inputfilename;
+        ret = fopen(inputfilename.ascii(), "r");
+    }
+
+    return ret;
+}
+
+bool DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate, 
                                    QDateTime pendDate) 
 {
-    QString ddurl = "http://datadirect.webservices.zap2it.com/tvlistings/xtvdService";
+    QString ferror;
+    QString tempfile;
+    FILE *fp = getInputFile(plineupsOnly, pstartDate, pendDate, ferror,
+                            tempfile);
 
-    if (plineupsOnly) 
-    {
-        pstartDate = QDateTime(QDate::currentDate().addDays(2),
-                               QTime::QTime(23, 59, 0));
-        pendDate = pstartDate;
-        pendDate = pendDate.addSecs(1);
-    }
-
-    QString startdatestr = pstartDate.toString(Qt::ISODate) + "Z";
-    QString enddatestr = pendDate.toString(Qt::ISODate) + "Z";
-
-    char ctempfilename[] = "/tmp/mythpostXXXXXX";
-    if (mkstemp(ctempfilename) == -1) 
-    {
-         perror("mkstemp");
-         VERBOSE(VB_IMPORTANT, "DDP: error creating temp files");
-         exit(-12);
-    }
-
-    QString tmpfilename = QString(ctempfilename);
-    QFile postfile(tmpfilename);
-
-    if (postfile.open(IO_WriteOnly)) 
-    {
-        QTextStream poststream(&postfile);
-        poststream << "<?xml version='1.0' encoding='utf-8'?>\n";
-        poststream << "<SOAP-ENV:Envelope\n";
-        poststream << "xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/'\n";
-        poststream << "xmlns:xsd='http://www.w3.org/2001/XMLSchema'\n";
-        poststream << "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'\n";
-        poststream << "xmlns:SOAP-ENC='http://schemas.xmlsoap.org/soap/encoding/'>\n";
-        poststream << "<SOAP-ENV:Body>\n";
-        poststream << "<ns1:download  xmlns:ns1='urn:TMSWebServices'>\n";
-        poststream << "<startTime xsi:type='xsd:dateTime'>";
-        poststream << startdatestr << "</startTime>\n";
-        poststream << "<endTime xsi:type='xsd:dateTime'>";
-        poststream << enddatestr << "</endTime>\n";
-        poststream << "</ns1:download>\n";
-        poststream << "</SOAP-ENV:Body>\n";
-        poststream << "</SOAP-ENV:Envelope>\n";
-    }
-
-    postfile.close();
-
-    QString command = QString("wget --http-user='%1' --http-passwd='%2' "
-                              "--post-file='%3' --header='Accept-Encoding:gzip'"
-                              " %4 --output-document=- | gzip -df")
-                             .arg(getUserID())
-                             .arg(getPassword())
-                             .arg(tmpfilename)
-                             .arg(ddurl);
-
-
-    FILE* fp = popen(command.ascii(), "r");
     if (fp == NULL) 
     {
-        VERBOSE(VB_GENERAL, "Failed to get data");
-        perror(command.ascii());
-        return;
+        VERBOSE(VB_IMPORTANT, QString("DDP: Failed to get data (%1) -- %2").
+                arg(ferror.ascii()).arg(strerror(errno)));
+        return false;
     }
 
     QFile f;
@@ -496,7 +538,7 @@ void DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate,
     if (!f.open(IO_ReadOnly, fp)) 
     {
        VERBOSE(VB_GENERAL,"Error opening DataDirect file");
-       return;
+       return false;
     }
     
     DDStructureParser ddhandler(*this);
@@ -506,24 +548,32 @@ void DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate,
     xmlsimplereader.setContentHandler(&ddhandler);
     xmlsimplereader.parse(xmlsource);
     f.close();
-    postfile.remove();
+
+    if (!tempfile.isNull())
+    {
+        QFile tmpfile(tempfile);
+        tmpfile.remove();
+    }
+    return true;
 }
 
-void DataDirectProcessor::grabLineupsOnly() 
+bool DataDirectProcessor::grabLineupsOnly() 
 {
+    bool ok = true;
     if ((lastrunuserid != getUserID()) || (lastrunpassword != getPassword())) 
     {
         lastrunuserid = getUserID();
         lastrunpassword = getPassword();
-        grabData(true, QDateTime::currentDateTime(),
-                 QDateTime::currentDateTime());
-    }   
+        ok = grabData(true, QDateTime::currentDateTime(),
+                      QDateTime::currentDateTime());
+    }
+    return ok;
 }   
 
-void DataDirectProcessor::grabAllData() 
+bool DataDirectProcessor::grabAllData() 
 {
-    grabData(false, QDateTime(QDate::currentDate()).addDays(-2),
-             QDateTime(QDate::currentDate()).addDays(15));
+    return grabData(false, QDateTime(QDate::currentDate()).addDays(-2),
+                    QDateTime(QDate::currentDate()).addDays(15));
 }
 
 void DataDirectProcessor::createATempTable(const QString &ptablename, 
@@ -555,12 +605,13 @@ void DataDirectProcessor::createTempTables()
             "postal char(6), device char(30) )";
     createATempTable("dd_lineup", table);  
 
-    table = "( lineupid char(100), stationid char(12), channel char(5) )";
+    table = "( lineupid char(100), stationid char(12), channel char(5), "
+        "channelMinor char(3) )";
     createATempTable("dd_lineupmap", table);
 
     table = "( stationid char(12), callsign char(10), stationname varchar(40), "
             "affiliate varchar(25), fccchannelnumber char(15), "
-            "channel char(5) )"; 
+            "channel char(5), channelMinor char(3) )"; 
     createATempTable ("dd_v_station", table);
 
     table = "( programid char(12), stationid char(12), scheduletime datetime, "
