@@ -14,7 +14,7 @@ class VideoOutputDX : public VideoOutput
 
     bool Init(int width, int height, float aspect, WId winid,
               int winx, int winy, int winw, int winh, WId embedid = 0);
-    void PrepareFrame(VideoFrame *buffer);
+    void PrepareFrame(VideoFrame *buffer, FrameScanType);
     void Show(FrameScanType );
 
     void InputChanged(int width, int height, float aspect);
@@ -26,7 +26,7 @@ class VideoOutputDX : public VideoOutput
 
     int GetRefreshRate(void);
 
-    void DrawUnusedRects(void);
+    void DrawUnusedRects(bool sync = true);
 
     void UpdatePauseFrame(void);
     void ProcessFrame(VideoFrame *frame, OSD *osd,
