@@ -48,36 +48,49 @@
 using namespace std;
 #include "vsync.h"
 
+bool tryingVideoSync = false;
+int VideoSync::m_forceskip = 0;
+
+#define TESTVIDEOSYNC(NAME) \
+    if (++m_forceskip > skip) \
+    { \
+        trial = new NAME (frame_interval, refresh_interval, interlaced); \
+        if (trial->TryInit()) \
+        { \
+	    m_forceskip = skip; \
+	    tryingVideoSync=false; \
+	    return trial; \
+        } \
+        delete trial; \
+    }
+
 VideoSync *VideoSync::BestMethod(int frame_interval, int refresh_interval,
                                  bool interlaced)
 {
     VideoSync *trial;
-    trial = new nVidiaVideoSync(frame_interval, refresh_interval, interlaced);
-    if (trial->TryInit())
-        return trial;
-    delete trial;
-    trial = new DRMVideoSync(frame_interval, refresh_interval, interlaced);
-    if (trial->TryInit())
-        return trial;
-    delete trial;
+
+    tryingVideoSync=true;
+    int skip=0;
+    if (m_forceskip!=0)
+    {
+        VERBOSE(VB_PLAYBACK, QString("A previous trial crashed,"
+		" skipping %1").arg(m_forceskip));
+    
+	skip=m_forceskip;
+	m_forceskip=0;
+    }
+    
+    TESTVIDEOSYNC(nVidiaVideoSync)
+    TESTVIDEOSYNC(DRMVideoSync)
 #ifdef USING_OPENGL_VSYNC
-    trial = new OpenGLVideoSync(frame_interval, refresh_interval, interlaced);
-    if (trial->TryInit())
-        return trial;
-    delete trial;
+    TESTVIDEOSYNC(OpenGLVideoSync)
 #endif
 #ifdef __linux__
-    trial = new RTCVideoSync(frame_interval, refresh_interval, interlaced);
-    if (trial->TryInit())
-        return trial;
-    delete trial;
+    TESTVIDEOSYNC(RTCVideoSync)
 #endif
-    trial = new BusyWaitVideoSync(frame_interval, refresh_interval, 
-                                  interlaced);
-    if (trial->TryInit())
-        return trial;
-    delete trial;
+    TESTVIDEOSYNC(BusyWaitVideoSync)
 
+    tryingVideoSync=false;
     return NULL;
 }
 
@@ -102,7 +115,9 @@ void VideoSync::SetFrameInterval(int fr, bool intr)
     m_interlaced = intr;
     if (m_interlaced && m_refresh_interval > m_frame_interval/2)
         m_interlaced = false; // can't display both fields at 2x rate
-    
+
+    VERBOSE(VB_PLAYBACK, QString("Set video sync frame interval to %1")
+                                 .arg(m_frame_interval));
 }
 
 void VideoSync::OffsetTimeval(struct timeval& tv, int offset)
@@ -267,20 +282,39 @@ void DRMVideoSync::WaitForFrame(int sync_delay)
     // Offset for externally-provided A/V sync delay
     OffsetTimeval(m_nexttrigger, sync_delay);
     
-    // Do the wait
     m_delay = CalcDelay();
-    
-    if (m_delay > 0) 
+    //cerr << "WaitForFrame at : " << m_delay;
+
+    // Always sync to the next retrace execpt when we are very late.
+    if (m_delay > -(m_refresh_interval/2)) 
     {
         drm_wait_vblank_t blank;
         blank.request.type = DRM_VBLANK_RELATIVE;
-        blank.request.sequence =
-            (int)(ceil((double)m_delay / m_refresh_interval));
+        blank.request.sequence = 1;
         drmWaitVBlank(m_dri_fd, &blank);
+        m_delay = CalcDelay();
+        // cerr << "\tDelay at sync: " << m_delay;
     }
+    //cerr  << endl;
 
+    if (m_delay > 0)
+    {
+        // Wait for any remaining retrace intervals in one pass.
+        int n = m_delay / m_refresh_interval + 1;
+
+        drm_wait_vblank_t blank;
+        blank.request.type = DRM_VBLANK_RELATIVE;
+        blank.request.sequence = n;
+        drmWaitVBlank(m_dri_fd, &blank);
+        m_delay = CalcDelay();
+        //cerr << "Wait " << n << " intervals. Count " << blank.request.sequence;
+        //cerr  << " Delay " << m_delay << endl;
+    }
+}
+
+void DRMVideoSync::AdvanceTrigger()
+{
     KeepPhase();
-    
     UpdateNexttrigger();
 }
 
@@ -345,14 +379,24 @@ void nVidiaVideoSync::WaitForFrame(int sync_delay)
     
     m_delay = CalcDelay();
 
+    // Always sync to the next retrace execpt when we are very late.
+    if (m_delay > -(m_refresh_interval/2))
+    {
+        dopoll();
+        m_delay = CalcDelay();
+    }
+
+    // Wait for any remaining retrace intervals.
     while (m_delay > 0)
     {
         dopoll();
         m_delay = CalcDelay();
     }
-    
+}
+
+void nVidiaVideoSync::AdvanceTrigger()
+{
     KeepPhase();
-    
     UpdateNexttrigger();
 }
 
@@ -387,14 +431,16 @@ bool OpenGLVideoSync::TryInit()
     }
 
     /* Look for GLX at all */
-    if (glXQueryExtension(m_display, NULL, NULL) == 0)
+    int ndummy;
+    if (glXQueryExtension(m_display, &ndummy, &ndummy) == 0)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: OpenGL extension not present.");
         return false;
     }
 
     /* Look for video sync extension */
-    const char *xt = glXQueryExtensionsString(m_display, 0);
+    const char *xt = glXQueryExtensionsString(m_display, DefaultScreen(m_display));
+    VERBOSE(VB_PLAYBACK, QString("OpenGLVideoSync: GLX extensions: %1").arg(xt));
     if (strstr(xt, "GLX_SGI_video_sync") == NULL)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: GLX Video Sync"
@@ -411,20 +457,20 @@ bool OpenGLVideoSync::TryInit()
     XSetWindowAttributes swa;
     Window w;
     
-    vis = glXChooseVisual(m_display, 0, attribList);
+    vis = glXChooseVisual(m_display, DefaultScreen(m_display), attribList);
     if (vis == NULL) 
     {
-	VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: No appropriate visual found");
+        VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: No appropriate visual found");
         return false;
     }
-    swa.colormap = XCreateColormap(m_display, RootWindow(m_display, 0),
+    swa.colormap = XCreateColormap(m_display, RootWindow(m_display, vis->screen),
                                    vis->visual, AllocNone);
     if (swa.colormap == 0)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Failed to create colormap");
         return false;
     }
-    w = XCreateWindow(m_display, RootWindow(m_display, 0), 0, 0, 1, 1,
+    w = XCreateWindow(m_display, RootWindow(m_display, vis->screen), 0, 0, 1, 1,
                       0, vis->depth, InputOutput, vis->visual, 
                       CWColormap, &swa);
     if (w == 0)
@@ -432,7 +478,7 @@ bool OpenGLVideoSync::TryInit()
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Failed to create dummy window");
         return false;
     }
-    m_context = glXCreateContext(m_display, vis, NULL, GL_TRUE);
+    m_context = glXCreateContext(m_display, vis, None, GL_TRUE);
     if (m_context == NULL)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Failed to create GLX context");
@@ -457,37 +503,39 @@ void OpenGLVideoSync::Start()
 
 void OpenGLVideoSync::WaitForFrame(int sync_delay)
 {
-    //if (sync_delay)
-        //cout << "Sync delay: " << sync_delay << endl;
-    // Offset for externally-provided A/V sync delay
+    unsigned int count, n;
+    int r;
+
     OffsetTimeval(m_nexttrigger, sync_delay);
-    
-    // Do the wait
+
     m_delay = CalcDelay();
-    //cout << "Delay " << m_delay;
+    //cerr << "WaitForFrame at : " << m_delay;
 
-    int n = 0;
-    while (m_delay > 0) 
+    // Always sync to the next retrace execpt when we are very late.
+    if (m_delay > -(m_refresh_interval/2)) 
     {
-        unsigned int count;
-        int r;
-        //int n = (int)ceil((double)m_delay / m_refresh_interval);
-        //cout << "Waiting " << n << " intervals" << endl;
         r = glXMakeCurrent(m_display, m_drawable, m_context);
-        //cout << "glxmc: " << r << endl;
         r = glXGetVideoSyncSGI(&count);
-        //cout << "glxgvs: " << r << " count " << count << endl;
-        // sleep for n frame intervals
-        //r = glXWaitVideoSyncSGI(n+1, (count+n)%(n+1) ,&count);
         r = glXWaitVideoSyncSGI(2, (count+1)%2 ,&count);
-        //cout << "glxwvs: " << r << " count " << count << endl;
         m_delay = CalcDelay();
-        n++;
+        //cerr << "\tDelay at sync: " << m_delay;
     }
+    //cerr << endl;
 
+    if (m_delay > 0)
+    {
+        // Wait for any remaining retrace intervals in one pass.
+        n = m_delay / m_refresh_interval + 1;
+        r = glXWaitVideoSyncSGI((n+1), (count+n)%(n+1), &count);
+        m_delay = CalcDelay();
+        //cerr << "Wait " << n << " intervals. Count " << count;
+        //cerr  << " Delay " << m_delay << endl;
+    }
+}
+
+void OpenGLVideoSync::AdvanceTrigger()
+{
     KeepPhase();
-    //cout << " after " << n << " " << m_delay << endl;
-    
     UpdateNexttrigger();
 }
 
@@ -536,7 +584,7 @@ bool RTCVideoSync::TryInit()
     if (ioctl(m_rtcfd, RTC_PIE_ON, 0) < 0)
     {
         VERBOSE(VB_PLAYBACK, QString("RTCVideoSync: Could not enable periodic "
-		"timer interrupts, %1.").arg(strerror(errno)));
+                "timer interrupts, %1.").arg(strerror(errno)));
         return false;
     }
     
@@ -555,6 +603,10 @@ void RTCVideoSync::WaitForFrame(int sync_delay)
         read(m_rtcfd, &rtcdata, sizeof(rtcdata));
         m_delay = CalcDelay();
     }
+}
+
+void RTCVideoSync::AdvanceTrigger()
+{
     UpdateNexttrigger();
 }
 #endif /* __linux__ */
@@ -603,7 +655,10 @@ void BusyWaitVideoSync::WaitForFrame(int sync_delay)
         if (cnt > 1)
             m_cheat -= 200;
     }
+}
 
+void BusyWaitVideoSync::AdvanceTrigger()
+{
     UpdateNexttrigger();
 }
 
@@ -626,10 +681,12 @@ void USleepVideoSync::WaitForFrame(int sync_delay)
     // Offset for externally-provided A/V sync delay
     OffsetTimeval(m_nexttrigger, sync_delay);
     
-    // Do the wait
     m_delay = CalcDelay();
     if (m_delay > 0)
         usleep(m_delay);
+}
 
+void USleepVideoSync::AdvanceTrigger()
+{
     UpdateNexttrigger();
 }
