@@ -9,9 +9,17 @@
 #include <algorithm>
 using namespace std;
 
-#include <sys/stat.h>
+#ifdef linux
 #include <sys/vfs.h>
+#else
+#include <sys/param.h>
+#include <sys/mount.h>
+#endif
+
+#include <sys/stat.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 
 #include "scheduler.h"
 
@@ -21,6 +29,7 @@ using namespace std;
 #include "libmyth/mythcontext.h"
 #include "mainserver.h"
 #include "remoteutil.h"
+#include "libmyth/mythdbcon.h"
 
 Scheduler::Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList, 
                      QSqlDatabase *ldb)
@@ -68,7 +77,7 @@ void Scheduler::verifyCards(void)
 
     thequery = "SELECT NULL FROM capturecard;";
 
-    QSqlQuery query = db->exec(thequery);
+    MSqlQuery query = db->exec(thequery);
 
     int numcards = -1;
     if (query.isActive())
@@ -78,7 +87,7 @@ void Scheduler::verifyCards(void)
     {
         cerr << "ERROR: no capture cards are defined in the database.\n";
         cerr << "Perhaps you should read the installation instructions?\n";
-        exit(1);
+        exit(5);
     }
 
     thequery = "SELECT sourceid,name FROM videosource ORDER BY sourceid;";
@@ -99,7 +108,7 @@ void Scheduler::verifyCards(void)
             thequery = QString("SELECT cardinputid FROM cardinput WHERE "
                                "sourceid = %1 ORDER BY cardinputid;")
                               .arg(source);
-            QSqlQuery subquery = db->exec(thequery);
+            MSqlQuery subquery = db->exec(thequery);
             
             if (!subquery.isActive() || subquery.numRowsAffected() <= 0)
                 cerr << query.value(1).toString() << " is defined, but isn't "
@@ -114,16 +123,26 @@ void Scheduler::verifyCards(void)
     }
 }
 
-bool Scheduler::CheckForChanges(void)
-{
-    return ScheduledRecording::hasChanged(db);
-}
-
 static inline bool Recording(ProgramInfo *p)
 {
     return (p->recstatus == rsRecording || p->recstatus == rsWillRecord);
 }
 
+static bool comp_overlap(ProgramInfo *a, ProgramInfo *b)
+{
+    if (a->startts != b->startts)
+        return a->startts < b->startts;
+    if (a->endts != b->endts)
+        return a->endts < b->endts;
+
+    // Note: the PruneOverlaps logic depends on the following
+    if (a->chanid != b->chanid)
+        return a->chanid < b->chanid;
+    if (a->inputid != b->inputid)
+        return a->inputid < b->inputid;
+    return a->recordid < b->recordid;
+}
+
 static bool comp_recstart(ProgramInfo *a, ProgramInfo *b)
 {
     if (a->recstartts != b->recstartts)
@@ -205,7 +224,7 @@ void Scheduler::FillEncoderFreeSpaceCache()
     }
 }
 
-bool Scheduler::FillRecordLists(void)
+bool Scheduler::FillRecordList(void)
 {
     QMutexLocker lockit(reclist_lock);
 
@@ -213,14 +232,20 @@ bool Scheduler::FillRecordLists(void)
     schedMoveHigher = (bool)gContext->GetNumSetting("SchedMoveHigher");
     schedTime = QDateTime::currentDateTime();
 
+    MythContext::KickDatabase(db);
+
     VERBOSE(VB_SCHEDULE, "PruneOldRecords...");
     PruneOldRecords();
     VERBOSE(VB_SCHEDULE, "AddNewRecords...");
     AddNewRecords();
 
+    VERBOSE(VB_SCHEDULE, "Sort by time...");
+    reclist.sort(comp_overlap);
+    VERBOSE(VB_SCHEDULE, "PruneOverlaps...");
+    PruneOverlaps();
+
     VERBOSE(VB_SCHEDULE, "Sort by priority...");
     reclist.sort(comp_priority);
-
     VERBOSE(VB_SCHEDULE, "BuildListMaps...");
     BuildListMaps();
 
@@ -241,25 +266,39 @@ bool Scheduler::FillRecordLists(void)
 
     VERBOSE(VB_SCHEDULE, "Sort by time...");
     reclist.sort(comp_recstart);
-
     VERBOSE(VB_SCHEDULE, "PruneRedundants...");
     PruneRedundants();
 
     return hasconflicts;
 }
 
+void Scheduler::FillRecordListFromDB(void)
+{
+    MSqlQuery query(QString::null, db);
+    query.prepare("CREATE TEMPORARY TABLE recordmatch "
+                  "(recordid int unsigned, chanid int unsigned, "
+                  " starttime datetime, INDEX (recordid));");
+    query.exec();
+    if (!query.isActive())
+    {
+        MythContext::DBError("FillRecordListFromDB", query);
+        return;
+    }
+
+    UpdateMatches(-1);
+    FillRecordList();
+}
+
 void Scheduler::FillRecordListFromMaster(void)
 {
-    vector<ProgramInfo *> remotelist;
-
-    RemoteGetAllPendingRecordings(remotelist);
-
-    vector<ProgramInfo *>::iterator pgiter = remotelist.begin();
+    ProgramList schedList(false);
+    schedList.FromScheduler();
 
     QMutexLocker lockit(reclist_lock);
 
-    for (; pgiter != remotelist.end(); pgiter++)
-        reclist.push_back(*pgiter);
+    ProgramInfo *p;
+    for (p = schedList.first(); p; p = schedList.next())
+        reclist.push_back(p);
 }
 
 void Scheduler::PrintList(bool onlyFutureRecordings)
@@ -353,7 +392,7 @@ bool Scheduler::ReactivateRecording(ProgramInfo *pginfo)
                 p->recstatus != rsWillRecord)
             {
                 p->reactivate = 1;
-                ScheduledRecording::signalChange(db);
+                Reschedule(0);
             }
             return true;
         }
@@ -387,6 +426,32 @@ void Scheduler::PruneOldRecords(void)
     }
 }
 
+void Scheduler::PruneOverlaps(void)
+{
+    ProgramInfo *lastp = NULL;
+
+    RecIter dreciter = reclist.begin();
+    while (dreciter != reclist.end())
+    {
+        ProgramInfo *p = *dreciter;
+        if (lastp == NULL || lastp->recordid == p->recordid ||
+            !lastp->IsSameTimeslot(*p))
+        {
+            lastp = p;
+            dreciter++;
+        }
+        else
+        {
+            int lpri = RecTypePriority(lastp->rectype);
+            int cpri = RecTypePriority(p->rectype);
+            if (lpri > cpri)
+                *lastp = *p;
+            delete p;
+            dreciter = reclist.erase(dreciter);
+        }
+    }
+}
+
 void Scheduler::BuildListMaps(void)
 {
     RecIter i = reclist.begin();
@@ -735,6 +800,13 @@ void Scheduler::getAllScheduled(QStringList &strList)
         (*i)->ToStringList(strList);
 }
 
+void Scheduler::Reschedule(int recordid) { 
+    reschedLock.lock(); 
+    reschedQueue.append(recordid); 
+    reschedWait.wakeOne();
+    reschedLock.unlock();
+}
+
 void Scheduler::RunScheduler(void)
 {
     int prerollseconds = 0;
@@ -759,30 +831,65 @@ void Scheduler::RunScheduler(void)
     bool firstRun = true;
 
     struct timeval fillstart, fillend;
+    float matchTime, placeTime;
 
     // wait for slaves to connect
     sleep(2);
 
+    Reschedule(-1);
+
     while (1)
     {
         curtime = QDateTime::currentDateTime();
         bool statuschanged = false;
 
-        if ((startIter == reclist.end() ||
-             curtime.secsTo((*startIter)->recstartts) > 30) &&
-            (CheckForChanges() ||
-             (lastupdate.date().day() != curtime.date().day())))
+      if ((startIter != reclist.end() &&
+           curtime.secsTo((*startIter)->recstartts) < 30))
+          sleep(1);
+      else
+      {
+        reschedLock.lock();
+        if (!reschedQueue.count())
+            reschedWait.wait(&reschedLock, 1000);
+        reschedLock.unlock();
+            
+        if (reschedQueue.count())
         {
-            VERBOSE(VB_GENERAL, "Found changes in the todo list.");
+            gettimeofday(&fillstart, NULL);
+            QString msg;
+            while (reschedQueue.count())
+            {
+                int recordid = reschedQueue.front();
+                reschedQueue.pop_front();
+                msg.sprintf("Reschedule requested for id %d.", recordid);
+                VERBOSE(VB_GENERAL, msg);
+                if (recordid != 0)
+                {
+                    if (recordid == -1)
+                        reschedQueue.clear();
+                    UpdateMatches(recordid);
+                }
+            }
+            gettimeofday(&fillend, NULL);
+
+            matchTime = ((fillend.tv_sec - fillstart.tv_sec ) * 1000000 +
+                         (fillend.tv_usec - fillstart.tv_usec)) / 1000000.0;
+
             FillEncoderFreeSpaceCache();
             gettimeofday(&fillstart, NULL);
-            FillRecordLists();
+            FillRecordList();
             gettimeofday(&fillend, NULL);
             PrintList();
-            VERBOSE(VB_GENERAL, QString("Scheduled %1 items in %2 seconds.")
-                    .arg(reclist.size())
-                    .arg(((fillend.tv_sec  - fillstart.tv_sec ) * 1000000 +
-                         (fillend.tv_usec - fillstart.tv_usec)) / 1000000.0));
+
+            placeTime = ((fillend.tv_sec - fillstart.tv_sec ) * 1000000 +
+                         (fillend.tv_usec - fillstart.tv_usec)) / 1000000.0;
+
+            msg.sprintf("Scheduled %d items in "
+                        "%.1f = %.2f match + %.2f place", reclist.size(),
+                        matchTime + placeTime, matchTime, placeTime);
+                         
+            VERBOSE(VB_GENERAL, msg);
+            gContext->LogEntry("scheduler", LP_INFO, "Scheduled items", msg);
 
             lastupdate = curtime;
             startIter = reclist.begin();
@@ -831,11 +938,14 @@ void Scheduler::RunScheduler(void)
                 firstRun = false;
             }
         }
+      }
 
         for ( ; startIter != reclist.end(); startIter++)
             if ((*startIter)->recstatus == rsWillRecord)
                 break;
 
+        curtime = QDateTime::currentDateTime();
+
         RecIter recIter = startIter;
         for ( ; recIter != reclist.end(); recIter++)
         {
@@ -877,7 +987,7 @@ void Scheduler::RunScheduler(void)
                     msg = QString("SUPPRESSED recording '%1' on channel"
                                   " %2 on cardid %3, sourceid %4.  Only"
                                   " %5 Megs of disk space available.")
-                        .arg(nextRecording->title.utf8())
+                        .arg(nextRecording->title.local8Bit())
                         .arg(nextRecording->chanid)
                         .arg(nextRecording->cardid)
                         .arg(nextRecording->sourceid)
@@ -896,7 +1006,7 @@ void Scheduler::RunScheduler(void)
                 msg = QString("SUPPRESSED recording \"%1\" on channel: "
                               "%2 on cardid: %3, sourceid %4. Tuner "
                               "is locked by an external application.")
-                    .arg(nextRecording->title.utf8())
+                    .arg(nextRecording->title.local8Bit())
                     .arg(nextRecording->chanid)
                     .arg(nextRecording->cardid)
                     .arg(nextRecording->sourceid);
@@ -960,11 +1070,13 @@ void Scheduler::RunScheduler(void)
             statuschanged = true;
 
             msg += QString(" \"%1\" on channel: %2 on cardid: %3, "
-                           "sourceid %4").arg(nextRecording->title.utf8())
+                           "sourceid %4").arg(nextRecording->title)
                 .arg(nextRecording->chanid)
                 .arg(nextRecording->cardid)
                 .arg(nextRecording->sourceid);
-            VERBOSE(VB_GENERAL, msg);
+            VERBOSE(VB_GENERAL, msg.local8Bit());
+            gContext->LogEntry("scheduler", LP_NOTICE, "Schedule Change", 
+                               msg.utf8());
         }
 
         if (statuschanged)
@@ -1045,8 +1157,6 @@ void Scheduler::RunScheduler(void)
                     idleSince = QDateTime();
             }
         }
-
-        sleep(1);
     }
 } 
 
@@ -1160,11 +1270,194 @@ void *Scheduler::SchedulerThread(void *param)
     return NULL;
 }
 
+void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from, 
+                                       QStringList &where)
+{
+    QString query;
+    MSqlQuery result;
+    QString qphrase;
+
+    query = QString("SELECT recordid,search,subtitle,description "
+                    "FROM record WHERE search <> %1 AND "
+                    "(recordid = %2 OR %3 = -1) ")
+        .arg(kNoSearch).arg(recordid).arg(recordid);
+
+    result = db->exec(query);
+    if (!result.isActive())
+    {
+        MythContext::DBError("BuildNewRecordsQueries", result);
+        return;
+    }
+
+    while (result.next())
+    {
+        qphrase = result.value(3).toString();
+        qphrase.replace("\'", "\\\'");
+
+        if (qphrase == "")
+        {
+            VERBOSE(VB_IMPORTANT, QString("Invalid search key in recordid %1")
+                                         .arg(result.value(0).toString()));
+            continue;
+        }
+
+        switch (result.value(1).toInt())
+        {
+        case kPowerSearch:
+            from << result.value(2).toString();
+            where << QString("record.recordid = %1 %2")
+                .arg(result.value(0).toString())
+                .arg(qphrase);
+            break;
+        case kTitleSearch:
+            from << "";
+            where << QString("record.recordid = %1 AND "
+                             "program.title LIKE '\%%2\%'")
+                .arg(result.value(0).toString())
+                .arg(qphrase);
+            break;
+        case kKeywordSearch:
+            from << "";
+            where << QString("record.recordid = %1 AND "
+                             "(program.title LIKE '\%%2\%' OR "
+                             " program.subtitle LIKE '\%%3\%' OR "
+                             " program.description LIKE '\%%4\%')")
+                .arg(result.value(0).toString())
+                .arg(qphrase).arg(qphrase).arg(qphrase);
+            break;
+        case kPeopleSearch:
+            from << ", people, credits";
+            where << QString("record.recordid = %1 AND "
+                             "people.name LIKE '\%%2\%' AND "
+                             "credits.person = people.person AND "
+                             "program.chanid = credits.chanid AND "
+                             "program.starttime = credits.starttime")
+                .arg(result.value(0).toString())
+                .arg(qphrase);
+            break;
+        default:
+            VERBOSE(VB_SCHEDULE, QString("Unknown RecSearchType "
+                                         "(%1) for recordid %2")
+                                         .arg(result.value(1).toInt())
+                                         .arg(result.value(0).toString()));
+            break;
+        }
+    }
+
+    if (recordid == -1 || from.count() == 0)
+    {
+        from << "";
+        where << QString("record.search = %1 AND "
+                         "(record.recordid = %2 OR %3 = -1) AND "
+                         "program.title = record.title ")
+            .arg(kNoSearch).arg(recordid).arg(recordid);
+    }
+}
+
+void Scheduler::UpdateMatches(int recordid) {
+    struct timeval dbstart, dbend;
+
+    MythContext::KickDatabase(db);
+    MSqlQuery query;
+    query.prepare("DELETE FROM recordmatch WHERE "
+                  "recordid = :RECORDID OR :RECORDID = -1;");
+    query.bindValue(":RECORDID", recordid);
+
+    query.exec();
+
+    if (!query.isActive())
+    {
+        MythContext::DBError("UpdateMatches", query);
+        return;
+    }
+
+    unsigned clause;
+    QStringList fromclauses, whereclauses;
+
+    BuildNewRecordsQueries(recordid, fromclauses, whereclauses);
+
+    if (print_verbose_messages & VB_SCHEDULE)
+    {
+        for (clause = 0; clause < fromclauses.count(); clause++)
+            cout << "Query " << clause << ": " << fromclauses[clause] 
+                 << "/" << whereclauses[clause] << endl;
+    }
+
+    for (clause = 0; clause < fromclauses.count(); clause++)
+    {
+        QString query = QString(
+"INSERT INTO recordmatch (recordid, chanid, starttime) "
+"SELECT record.recordid, program.chanid, program.starttime "
+"FROM record, program ") + fromclauses[clause] + QString(
+" INNER JOIN channel ON (channel.chanid = program.chanid) "
+"WHERE ") + whereclauses[clause] + QString(" AND "
+"((record.type = %1 " // allrecord
+"OR record.type = %2 " // findonerecord
+"OR record.type = %3 " // finddailyrecord
+"OR record.type = %4) " // findweeklyrecord
+" OR "
+" ((record.station = channel.callsign) " // channel matches
+"  AND "
+"  ((record.type = %5) " // channelrecord
+"   OR"
+"   ((TIME_TO_SEC(record.starttime) = TIME_TO_SEC(program.starttime)) " // timeslot matches
+"    AND "
+"    ((record.type = %6) " // timeslotrecord
+"     OR"
+"     ((DAYOFWEEK(record.startdate) = DAYOFWEEK(program.starttime) "
+"      AND "
+"      ((record.type = %7) " // weekslotrecord
+"       OR"
+"       ((TO_DAYS(record.startdate) = TO_DAYS(program.starttime)) " // date matches
+"        AND "
+"        (TIME_TO_SEC(record.endtime) = TIME_TO_SEC(program.endtime)) "
+"        AND "
+"        (TO_DAYS(record.enddate) = TO_DAYS(program.endtime)) "
+"        )"
+"       )"
+"      )"
+"     )"
+"    )"
+"   )"
+"  )"
+" )"
+") ")
+            .arg(kAllRecord)
+            .arg(kFindOneRecord)
+            .arg(kFindDailyRecord)
+            .arg(kFindWeeklyRecord)
+            .arg(kChannelRecord)
+            .arg(kTimeslotRecord)
+            .arg(kWeekslotRecord);
+
+        VERBOSE(VB_SCHEDULE, QString(" |-- Start DB Query %1...").arg(clause));
+
+        gettimeofday(&dbstart, NULL);
+        MSqlQuery result = db->exec(query);
+        gettimeofday(&dbend, NULL);
+
+        if (!result.isActive())
+        {
+            MythContext::DBError("UpdateMatches", result);
+            return;
+        }
+
+        VERBOSE(VB_SCHEDULE, QString(" |-- %1 results in %2 sec.")
+                .arg(result.size())
+                .arg(((dbend.tv_sec  - dbstart.tv_sec) * 1000000 +
+                      (dbend.tv_usec - dbstart.tv_usec)) / 1000000.0));
+
+    }
+
+    VERBOSE(VB_SCHEDULE, " +-- Done.");
+}
+
 void Scheduler::AddNewRecords(void) {
+
+    struct timeval dbstart, dbend;
+
     QMap<RecordingType, int> recTypeRecPriorityMap;
     RecList tmpList;
-    ProgramInfo *lastp = NULL;
-    QMap<int, bool> allowmap;
 
     QMap<int, bool> cardMap;
     QMap<int, EncoderLink *>::Iterator enciter = m_tvList->begin();
@@ -1175,6 +1468,69 @@ void Scheduler::AddNewRecords(void) {
             cardMap[enc->getCardId()] = true;
     }
 
+    QMap<int, bool> tooManyMap;
+    bool checkTooMany = false;
+
+    MSqlQuery rlist(QString::null, db);
+    rlist.prepare("SELECT recordid,title,maxepisodes,maxnewest FROM record;");
+
+    rlist.exec();
+
+    if (!rlist.isActive())
+    {
+        MythContext::DBError("CheckTooMany", rlist);
+        return;
+    }
+
+    while (rlist.next())
+    {
+        int recid = rlist.value(0).toInt();
+        QString qtitle = QString::fromUtf8(rlist.value(1).toString());
+        int maxEpisodes = rlist.value(2).toInt();
+        int maxNewest = rlist.value(3).toInt();
+
+        tooManyMap[recid] = false;
+
+        if (maxEpisodes && !maxNewest)
+        {
+            MSqlQuery epicnt(QString::null, db);
+
+            epicnt.prepare("SELECT count(*) FROM recorded "
+                           "WHERE title = :TITLE;");
+            epicnt.bindValue(":TITLE", qtitle.utf8());
+
+            epicnt.exec();
+
+            if (epicnt.isActive() && epicnt.numRowsAffected() > 0)
+            {
+                epicnt.next();
+
+                if (epicnt.value(0).toInt() >= maxEpisodes)
+                {
+                    tooManyMap[recid] = true;
+                    checkTooMany = true;
+                }
+            }
+        }
+    }
+
+    QString progfindid = QString(
+"(CASE record.type "
+"  WHEN %1 "
+"   THEN to_days(date_sub(program.starttime, interval "
+"                time_format(record.findtime, '%H:%i') hour_minute)) "
+"  WHEN %2 "
+"   THEN floor((to_days(date_sub(program.starttime, interval "
+"               time_format(record.findtime, '%H:%i') hour_minute)) - "
+"               record.findday)/7) * 7 + record.findday "
+"  WHEN %3 "
+"   THEN record.findid "
+"  ELSE 0 "
+" END) ")
+        .arg(kFindDailyRecord)
+        .arg(kFindWeeklyRecord)
+        .arg(kOverrideRecord);
+
     QString query = QString(
 "SELECT DISTINCT channel.chanid, channel.sourceid, "
 "program.starttime, program.endtime, "
@@ -1191,9 +1547,15 @@ void Scheduler::AddNewRecords(void) {
 "program.previouslyshown, record.recgroup, record.dupmethod, "
 "channel.commfree, capturecard.cardid, "
 "cardinput.cardinputid, UPPER(cardinput.shareable) = 'Y' AS shareable, "
-"program.seriesid, program.programid "
-"FROM record "
-" INNER JOIN program ON (program.title = record.title) "
+"program.seriesid, program.programid, program.category_type, "
+"program.airdate, program.stars, program.originalairdate, record.inactive, ")
++ progfindid + QString(
+
+"FROM recordmatch "
+
+" INNER JOIN record ON (recordmatch.recordid = record.recordid) "
+" INNER JOIN program ON (recordmatch.chanid = program.chanid AND "
+"                        recordmatch.starttime = program.starttime) "
 " INNER JOIN channel ON (channel.chanid = program.chanid) "
 " INNER JOIN cardinput ON (channel.sourceid = cardinput.sourceid) "
 " INNER JOIN capturecard ON (capturecard.cardid = cardinput.cardid) "
@@ -1203,9 +1565,13 @@ void Scheduler::AddNewRecords(void) {
 "    program.title = oldrecorded.title "
 "     AND "
 "     ( "
-"      (program.programid <> '' AND program.programid NOT LIKE 'SH%0000' "
+"      (program.programid <> '' AND NOT "
+"       (program.category_type = 'series' AND program.programid LIKE '%0000') "
 "       AND program.programid = oldrecorded.programid) "
 "      OR "
+"      (oldrecorded.findid <> 0 AND "
+"        oldrecorded.findid = ") + progfindid + QString(") "
+"      OR "
 "      ( "
 "       (program.programid = '' OR oldrecorded.programid = '') "
 "       AND "
@@ -1223,9 +1589,13 @@ void Scheduler::AddNewRecords(void) {
 "    program.title = recorded.title "
 "     AND "
 "     ( "
-"      (program.programid <> '' AND program.programid NOT LIKE 'SH%0000' "
+"      (program.programid <> '' AND NOT "
+"       (program.category_type = 'series' AND program.programid LIKE '%0000') "
 "       AND program.programid = recorded.programid) "
 "      OR "
+"      (recorded.findid <> 0 AND "
+"        recorded.findid = ") + progfindid + QString(") "
+"      OR "
 "      ( "
 "       (program.programid = '' OR recorded.programid = '') "
 "       AND "
@@ -1237,48 +1607,12 @@ void Scheduler::AddNewRecords(void) {
 "      ) "
 "     ) "
 "  ) ");
-    query += QString(
-"WHERE "
-"((record.type = %1 " // allrecord
-"OR record.type = %2) " // findonerecord
-" OR "
-" ((record.station = channel.callsign) " // channel matches
-"  AND "
-"  ((record.type = %3) " // channelrecord
-"   OR"
-"   ((TIME_TO_SEC(record.starttime) = TIME_TO_SEC(program.starttime)) " // timeslot matches
-"    AND "
-"    ((record.type = %4) " // timeslotrecord
-"     OR"
-"     ((DAYOFWEEK(record.startdate) = DAYOFWEEK(program.starttime) "
-"      AND "
-"      ((record.type = %5) " // weekslotrecord
-"       OR"
-"       ((TO_DAYS(record.startdate) = TO_DAYS(program.starttime)) " // date matches
-"        AND "
-"        (TIME_TO_SEC(record.endtime) = TIME_TO_SEC(program.endtime)) "
-"        AND "
-"        (TO_DAYS(record.enddate) = TO_DAYS(program.endtime)) "
-"        )"
-"       )"
-"      )"
-"     )"
-"    )"
-"   )"
-"  )"
-" )"
-") "
-// Note: the overlap code below depends on this ordering
-"ORDER BY program.starttime,program.endtime,program.chanid,"
-"cardinput.cardinputid,record.recordid;")
-        .arg(kAllRecord)
-        .arg(kFindOneRecord)
-        .arg(kChannelRecord)
-        .arg(kTimeslotRecord)
-        .arg(kWeekslotRecord);
 
-    VERBOSE(VB_SCHEDULE, " |-- Start DB Query...");
-    QSqlQuery result = db->exec(query);
+    VERBOSE(VB_SCHEDULE, QString(" |-- Start DB Query..."));
+
+    gettimeofday(&dbstart, NULL);
+    MSqlQuery result = db->exec(query);
+    gettimeofday(&dbend, NULL);
 
     if (!result.isActive())
     {
@@ -1286,8 +1620,11 @@ void Scheduler::AddNewRecords(void) {
         return;
     }
 
-    VERBOSE(VB_SCHEDULE, QString(" |-- Processing %1 results...")
-                          .arg(result.size()));
+    VERBOSE(VB_SCHEDULE, QString(" |-- %1 results in %2 sec. Processing...")
+            .arg(result.size())
+            .arg(((dbend.tv_sec  - dbstart.tv_sec) * 1000000 +
+                  (dbend.tv_usec - dbstart.tv_usec)) / 1000000.0));
+
     while (result.next())
     {
         // Don't bother if card isn't on-line of end time has already passed
@@ -1308,7 +1645,7 @@ void Scheduler::AddNewRecords(void) {
         p->subtitle = QString::fromUtf8(result.value(5).toString());
         p->description = QString::fromUtf8(result.value(6).toString());
         p->chanstr = result.value(7).toString();
-        p->chansign = result.value(8).toString();
+        p->chansign = QString::fromUtf8(result.value(8).toString());
         p->channame = QString::fromUtf8(result.value(9).toString());
         p->category = QString::fromUtf8(result.value(11).toString());
         p->recpriority = result.value(12).toInt();
@@ -1319,12 +1656,25 @@ void Scheduler::AddNewRecords(void) {
 
         p->recstartts = result.value(18).toDateTime();
         p->recendts = recendts;
+        p->repeat = result.value(20).toInt();
+        p->recgroup = result.value(21).toString();
         p->chancommfree = result.value(23).toInt();
         p->cardid = cardid;
         p->inputid = result.value(25).toInt();
         p->shareable = result.value(26).toInt();
         p->seriesid = result.value(27).toString();
         p->programid = result.value(28).toString();
+        p->catType = result.value(29).toString();
+        p->year = result.value(30).toString();
+        p->stars =  result.value(31).toDouble();
+
+        if(result.value(32).isNull())
+            p->originalAirDate = p->startts.date();
+        else
+            p->originalAirDate = QDate::fromString(result.value(30).toString(), Qt::ISODate);
+
+        bool inactive = result.value(33).toInt();
+        p->findid = result.value(34).toInt();
 
         if (!recTypeRecPriorityMap.contains(p->rectype))
             recTypeRecPriorityMap[p->rectype] = 
@@ -1333,14 +1683,11 @@ void Scheduler::AddNewRecords(void) {
 
         if (p->recstartts >= p->recendts)
         {
-            // pre/post-roll are invalid so ignore
+            // start/end-offsets are invalid so ignore
             p->recstartts = p->startts;
             p->recendts = p->endts;
         }
 
-        p->repeat = result.value(20).toInt();
-        p->recgroup = result.value(21).toString();
-
         p->schedulerid = 
             p->startts.toString() + "_" + p->chanid;
 
@@ -1373,9 +1720,7 @@ void Scheduler::AddNewRecords(void) {
             continue;
 
         // Check for rsTooManyRecordings
-        if (!allowmap.contains(p->recordid))
-            allowmap[p->recordid] = p->AllowRecordingNewEpisodes(db);
-        if (!p->reactivate && !allowmap[p->recordid])
+        if (checkTooMany && tooManyMap[p->recordid] && !p->reactivate)
             p->recstatus = rsTooManyRecordings;
 
         // Check for rsCurrentRecording and rsPreviousRecording
@@ -1386,27 +1731,20 @@ void Scheduler::AddNewRecords(void) {
                  !p->reactivate &&
                  !(p->dupmethod & kDupCheckNone))
         {
-            if (p->dupin & kDupsInOldRecorded &&
+            if (p->dupin == kDupsNewEpi && p->repeat)
+                p->recstatus = rsRepeat;
+        
+            if (((p->dupin & kDupsInOldRecorded) || (p->dupin == kDupsNewEpi)) &&
                 result.value(10).toInt())
                 p->recstatus = rsPreviousRecording;
-            if (p->dupin & kDupsInRecorded &&
+            
+            if (((p->dupin & kDupsInRecorded) || (p->dupin == kDupsNewEpi)) &&
                 result.value(14).toInt())
                 p->recstatus = rsCurrentRecording;
         }
 
-        // Check for overlap against last non-overlap
-        if (lastp == NULL || lastp->recordid == p->recordid ||
-            !lastp->IsSameTimeslot(*p))
-            lastp = p;
-        else
-        {
-            int lpri = RecTypePriority(lastp->rectype);
-            int cpri = RecTypePriority(p->rectype);
-            if (lpri > cpri)
-                *lastp = *p;
-            delete p;
-            continue;
-        }
+        if (inactive)
+            p->recstatus = rsInactive;
 
         tmpList.push_back(p);
     }
@@ -1439,13 +1777,13 @@ void Scheduler::findAllScheduledPrograms(list<ProgramInfo *> &proglist)
 "record.subtitle, record.description, record.recpriority, record.type, "
 "channel.name, record.recordid, record.recgroup, record.dupin, "
 "record.dupmethod, channel.commfree, channel.channum, record.station, "
-"record.seriesid, record.programid "
+"record.seriesid, record.programid, record.category, record.findid "
 "FROM record "
 "LEFT JOIN channel ON channel.callsign = record.station "
 "GROUP BY recordid "
 "ORDER BY title ASC;");
 
-    QSqlQuery result = db->exec(query);
+    MSqlQuery result = db->exec(query);
 
     if (result.isActive() && result.numRowsAffected() > 0)
         while (result.next()) 
@@ -1493,9 +1831,11 @@ void Scheduler::findAllScheduledPrograms(list<ProgramInfo *> &proglist)
             proginfo->chanstr = result.value(16).toString();
             if (proginfo->chanstr.isNull())
                 proginfo->chanstr = "";
-            proginfo->chansign = result.value(17).toString();
+            proginfo->chansign = QString::fromUtf8(result.value(17).toString());
             proginfo->seriesid = result.value(18).toString();
             proginfo->programid = result.value(19).toString();
+            proginfo->category = result.value(20).toString();
+            proginfo->findid = result.value(21).toInt();
             
             proginfo->recstartts = proginfo->startts;
             proginfo->recendts = proginfo->endts;
