@@ -141,9 +141,11 @@ ProgramRecPriority::ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent
 
     updateBackground();
 
+    MythContext::KickDatabase(db);
     FillList();
     sortType = (SortType)gContext->GetNumSetting("ProgramRecPrioritySorting", 
                                                  (int)byTitle);
+    reverseSort = gContext->GetNumSetting("ProgramRecPriorityReverse", 0);
 
     SortList(); 
     inList = inData = 0;
@@ -185,11 +187,15 @@ void ProgramRecPriority::keyPressEvent(QKeyEvent *e)
                 changeRecPriority(1);
             else if (action == "LEFT")
                 changeRecPriority(-1);
+            else if ((action == "PAUSE") || (action == "PLAYBACK"))
+                deactivate();
             else if (action == "ESCAPE")
             {
                 saveRecPriority();
                 gContext->SaveSetting("ProgramRecPrioritySorting",
                                       (int)sortType);
+                gContext->SaveSetting("ProgramRecPriorityReverse",
+                                      (int)reverseSort);
                 done(MythDialog::Accepted);
             }
             else if (action == "1")
@@ -197,23 +203,50 @@ void ProgramRecPriority::keyPressEvent(QKeyEvent *e)
                 if (sortType != byTitle)
                 {
                     sortType = byTitle;
-                    SortList();
-                    update(fullRect);
+                    reverseSort = false;
                 }
+                else
+                {
+                    reverseSort = !reverseSort;
+                }
+                SortList();
+                update(fullRect);
             }
             else if (action == "2")
             {
                 if (sortType != byRecPriority)
                 {
                     sortType = byRecPriority;
-                    SortList();
-                    update(fullRect);
+                    reverseSort = false;
                 }
+                else
+                {
+                    reverseSort = !reverseSort;
+                }
+                SortList();
+                update(fullRect);
+            }
+            else if (action == "4")
+            {
+                if (sortType != byRecType)
+                {
+                    sortType = byRecType;
+                    reverseSort = false;
+                }
+                else
+                {
+                    reverseSort = !reverseSort;
+                }
+                SortList();
+                update(fullRect);
             }
             else if (action == "PREVVIEW" || action == "NEXTVIEW")
             {
+                reverseSort = false;
                 if (sortType == byTitle)
                     sortType = byRecPriority;
+                else if (sortType == byRecPriority)
+                    sortType = byRecType;
                 else
                     sortType = byTitle;
                 SortList();
@@ -440,7 +473,7 @@ void ProgramRecPriority::edit(void)
             recid = rec->getRecordID(db);
 
         thequery = QString(
-                   "SELECT recpriority, type FROM record WHERE recordid = %1;")
+                   "SELECT recpriority, type, inactive FROM record WHERE recordid = %1;")
                            .arg(recid);
         QSqlQuery query = db->exec(thequery);
 
@@ -450,6 +483,7 @@ void ProgramRecPriority::edit(void)
                 query.next();
                 int recPriority = query.value(0).toInt();
                 int rectype = query.value(1).toInt();
+                int inactive = query.value(2).toInt();
 
                 int cnt;
                 QMap<QString, ProgramRecPriorityInfo>::Iterator it;
@@ -461,7 +495,7 @@ void ProgramRecPriority::edit(void)
                      cnt++, ++it);
                 progInfo = &(it.data());
            
-                int rtRecPriors[8];
+                int rtRecPriors[10];
                 rtRecPriors[0] = gContext->GetNumSetting("SingleRecordRecPrior", 0);
                 rtRecPriors[1] = gContext->GetNumSetting("TimeslotRecordRecPrior", 0);
                 rtRecPriors[2] = gContext->GetNumSetting("ChannelRecordRecPrior", 0);
@@ -470,17 +504,21 @@ void ProgramRecPriority::edit(void)
                 rtRecPriors[5] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
                 rtRecPriors[6] = gContext->GetNumSetting("OverrideRecordRecPrior", 0);
                 rtRecPriors[7] = gContext->GetNumSetting("OverrideRecordRecPrior", 0);
+                rtRecPriors[8] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
+                rtRecPriors[9] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
 
                 // set the recording priorities of that program 
                 progInfo->recpriority = recPriority;
                 progInfo->recType = (RecordingType)rectype;
                 progInfo->recTypeRecPriority = rtRecPriors[progInfo->recType-1];
-
                 // also set the origRecPriorityData with new recording 
                 // priority so we don't save to db again when we exit
                 QString key = progInfo->MakeUniqueKey(); 
                 origRecPriorityData[key] = progInfo->recpriority;
 
+                // also set the active/inactive state
+                progInfo->recstatus = inactive ? rsInactive : rsWillRecord;
+
                 SortList();
             }
             else
@@ -515,18 +553,73 @@ void ProgramRecPriority::edit(void)
     }
 }
 
-void ProgramRecPriority::upcoming(void)
+void ProgramRecPriority::deactivate(void)
 {
     if (!curitem)
         return;
 
     ProgramRecPriorityInfo *rec = curitem;
 
-    ProgLister *pl = new ProgLister(plTitle, rec->title,
-                                   QSqlDatabase::database(),
-                                   gContext->GetMainWindow(), "proglist");
-    pl->exec();
-    delete pl;
+    MythContext::KickDatabase(db);
+
+    if (rec)
+    {
+        QString thequery;
+
+        thequery = QString("SELECT inactive FROM record WHERE recordid = %1")
+                           .arg(rec->recordid);
+
+        QSqlQuery query = db->exec(thequery);
+
+        int inactive = 0;
+        if (query.isActive())
+            if (query.numRowsAffected() > 0)
+            {
+                query.next();
+                inactive = query.value(0).toInt();
+                if (inactive)
+                    inactive = 0;
+                else
+                    inactive = 1;
+
+                QString theupdatequery;
+                theupdatequery = QString("UPDATE record SET inactive = %1 WHERE recordid = %2")
+                                         .arg(inactive).arg(rec->recordid);
+
+                QSqlQuery uquery = db->exec(theupdatequery);
+
+                if (uquery.isActive())
+                {
+                    ScheduledRecording::signalChange(0);
+                    int cnt;
+                    QMap<QString, ProgramRecPriorityInfo>::Iterator it;
+                    ProgramRecPriorityInfo *progInfo;
+
+                    // iterate through programData till we hit the line where
+                    // the cursor currently is
+                    for (cnt = 0, it = programData.begin(); cnt < inList+inData;
+                         cnt++, ++it);
+                    progInfo = &(it.data());
+                    progInfo->recstatus = inactive ? rsInactive : rsWillRecord;
+                } else
+                    MythContext::DBError("Update recording schedule inactive query", uquery);
+            }
+
+        QPainter p(this);
+        updateInfo(&p);
+        update(fullRect);
+    }
+}
+
+void ProgramRecPriority::upcoming(void)
+{
+    if (!curitem)
+        return;
+
+    ScheduledRecording record;
+
+    record.loadByID(db, curitem->recordid);
+    record.runProgList();
 }
 
 void ProgramRecPriority::changeRecPriority(int howMuch) 
@@ -573,7 +666,7 @@ void ProgramRecPriority::saveRecPriority(void)
 
 void ProgramRecPriority::FillList(void)
 {
-    int cnt = 999, rtRecPriors[8];
+    int cnt = 999, rtRecPriors[10];
     vector<ProgramInfo *> recordinglist;
 
     programData.clear();
@@ -608,55 +701,42 @@ void ProgramRecPriority::FillList(void)
     rtRecPriors[5] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
     rtRecPriors[6] = gContext->GetNumSetting("OverrideRecordRecPriority", 0);
     rtRecPriors[7] = gContext->GetNumSetting("OverrideRecordRecPriority", 0);
+    rtRecPriors[8] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
+    rtRecPriors[9] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
     
     // get channel recording priorities and recording types associated with each
     // program from db
     // (hope this is ok to do here, it's so much lighter doing
     // it all at once than once per program)
-    QString query = QString("SELECT record.title, record.chanid, "
+    QString query = QString("SELECT recordid, record.title, record.chanid, "
                             "record.starttime, record.startdate, "
-                            "record.type, channel.recpriority "
+                            "record.type, channel.recpriority,  "
+                            "record.inactive "
                             "FROM record "
                             "LEFT JOIN channel ON "
                             "(record.chanid = channel.chanid);");
 
     QSqlQuery result = db->exec(query);
    
-//    cerr << "db query returned " << result.numRowsAffected() << " programs";
-//    cerr << endl; 
     int matches = 0;
 
     if (result.isActive() && result.numRowsAffected() > 0)
     {
         while (result.next()) 
         {
-            QString title = QString::fromUtf8(result.value(0).toString());
-            QString chanid = result.value(1).toString();
-            QString tempTime = result.value(2).toString();
-            QString tempDate = result.value(3).toString();
-            RecordingType recType = (RecordingType)result.value(4).toInt();
-            int channelRecPriority = result.value(5).toInt();
+            int recordid = result.value(0).toInt();
+            QString title = QString::fromUtf8(result.value(1).toString());
+            QString chanid = result.value(2).toString();
+            QString tempTime = result.value(3).toString();
+            QString tempDate = result.value(4).toString();
+            RecordingType recType = (RecordingType)result.value(5).toInt();
+            int channelRecPriority = result.value(6).toInt();
             int recTypeRecPriority = rtRecPriors[recType-1];
-              
-            // this is so kludgy
-            // since we key off of title+chanid+startts we have
-            // to copy what RemoteGetAllScheduledRecordings()
-            // does so the keys will match 
-            QDateTime startts; 
-            if (recType == kSingleRecord || recType == kTimeslotRecord ||
-                recType == kWeekslotRecord || recType == kOverrideRecord ||
-                recType == kDontRecord)
-                startts = QDateTime::fromString(tempDate + ":" + tempTime,
-                                                Qt::ISODate);
-            else
-            {
-                startts = QDateTime::currentDateTime();
-                startts.setTime(QTime(0,0));
-            }
-    
-            // make a key for each program
-            QString keyA = title + ":" + chanid + ":" +
-                           startts.toString(Qt::ISODate);
+            int inactive = result.value(7).toInt();
+
+            if (recType == kAllRecord || recType == kFindOneRecord ||
+                recType == kFindDailyRecord || recType == kFindWeeklyRecord)
+                channelRecPriority = 0;
 
             // find matching program in programData and set
             // channelRecPriority, recTypeRecPriority and recType
@@ -664,12 +744,16 @@ void ProgramRecPriority::FillList(void)
             for (it = programData.begin(); it != programData.end(); ++it)
             {
                 ProgramRecPriorityInfo *progInfo = &(it.data());
-                QString keyB = progInfo->MakeUniqueKey();
-                if (keyA == keyB)
+
+                if (progInfo->recordid == recordid)
                 {
+                    progInfo->sortTitle = progInfo->title;
+                    progInfo->sortTitle.remove(QRegExp(tr("^(The |A |An )")));
+
                     progInfo->channelRecPriority = channelRecPriority;
                     progInfo->recTypeRecPriority = recTypeRecPriority;
                     progInfo->recType = recType;
+                    progInfo->recstatus = inactive ? rsInactive : rsWillRecord;
                     matches++;
                     break;
                 }
@@ -679,7 +763,21 @@ void ProgramRecPriority::FillList(void)
     else
         MythContext::DBError("Get program recording priorities query", query);
 
-//    cerr << matches << " matches made" << endl;
+    recMatch.clear();
+    ProgramList schedList;
+    schedList.FromScheduler();
+    QDateTime now = QDateTime::currentDateTime();
+
+    ProgramInfo *s;
+    for (s = schedList.first(); s; s = schedList.next())
+    {
+        if (s->recendts > now)
+        {
+            listMatch[s->recordid]++;
+            if (s->recstatus == rsWillRecord || s->recstatus == rsRecording)
+                recMatch[s->recordid]++;
+        }
+    }
 }
 
 typedef struct RecPriorityInfo 
@@ -691,15 +789,26 @@ typedef struct RecPriorityInfo
 class titleSort 
 {
     public:
+        titleSort(bool reverseSort = false) {m_reverse = reverseSort;}
+
         bool operator()(const RecPriorityInfo a, const RecPriorityInfo b) 
         {
-            return (a.prog->title > b.prog->title);
+            if (m_reverse)
+                return (a.prog->sortTitle < b.prog->sortTitle);
+            else
+                return (a.prog->sortTitle > b.prog->sortTitle);
         }
+
+    private:
+        bool m_reverse;
 };
 
 class programRecPrioritySort 
 {
     public:
+        programRecPrioritySort(bool reverseSort = false)
+                               {m_reverse = reverseSort;}
+
         bool operator()(const RecPriorityInfo a, const RecPriorityInfo b) 
         {
             int finalA = a.prog->recpriority + 
@@ -710,9 +819,54 @@ class programRecPrioritySort
                          b.prog->recTypeRecPriority;
 
             if (finalA == finalB)
-                return (a.prog->title > b.prog->title);
-            return (finalA < finalB);
+            {
+                int typeA = RecTypePriority(a.prog->recType);
+                int typeB = RecTypePriority(b.prog->recType);
+                if (typeA == typeB)
+                    if (m_reverse)
+                        return (a.prog->sortTitle < b.prog->sortTitle);
+                    else
+                        return (a.prog->sortTitle > b.prog->sortTitle);
+
+                if (m_reverse)
+                    return (typeA < typeB);
+                else
+                    return (typeA > typeB);
+            }
+            if (m_reverse)
+                return (finalA > finalB);
+            else
+                return (finalA < finalB);
         }
+
+    private:
+        bool m_reverse;
+};
+
+class programRecTypeSort 
+{
+    public:
+        programRecTypeSort(bool reverseSort = false)
+                               {m_reverse = reverseSort;}
+
+        bool operator()(const RecPriorityInfo a, const RecPriorityInfo b) 
+        {
+            int typeA = RecTypePriority(a.prog->recType);
+            int typeB = RecTypePriority(b.prog->recType);
+            if (typeA == typeB)
+                if (m_reverse)
+                    return (a.prog->sortTitle < b.prog->sortTitle);
+                else
+                    return (a.prog->sortTitle > b.prog->sortTitle);
+
+            if (m_reverse)
+                return (typeA < typeB);
+            else
+                return (typeA > typeB);
+        }
+
+    private:
+        bool m_reverse;
 };
 
 void ProgramRecPriority::SortList() 
@@ -739,11 +893,29 @@ void ProgramRecPriority::SortList()
     // sort sortedList
     switch(sortType) 
     {
-        case byTitle : sort(sortedList.begin(), sortedList.end(), titleSort());
-                       break;
-        case byRecPriority : sort(sortedList.begin(), sortedList.end(), 
-                           programRecPrioritySort());
-                      break;
+        case byTitle :
+                 if (reverseSort)
+                     sort(sortedList.begin(), sortedList.end(),
+                          titleSort(true));
+                 else
+                     sort(sortedList.begin(), sortedList.end(), titleSort());
+                 break;
+        case byRecPriority :
+                 if (reverseSort)
+                     sort(sortedList.begin(), sortedList.end(), 
+                          programRecPrioritySort(true));
+                 else
+                     sort(sortedList.begin(), sortedList.end(), 
+                          programRecPrioritySort());
+                 break;
+        case byRecType :
+                 if (reverseSort)
+                     sort(sortedList.begin(), sortedList.end(), 
+                          programRecTypeSort(true));
+                 else
+                     sort(sortedList.begin(), sortedList.end(), 
+                          programRecTypeSort());
+                 break;
     }
 
     programData.clear();
@@ -837,25 +1009,31 @@ void ProgramRecPriority::updateList(QPainter *p)
                             ltype->SetItemCurrent(cnt);
                         }
 
-                        ltype->SetItemText(cnt, 1, tempSubTitle);
+                        ltype->SetItemText(cnt, 1, progInfo->RecTypeChar());
+                        ltype->SetItemText(cnt, 2, tempSubTitle);
 
-                        if (progRecPriority >= 0)
-                            ltype->SetItemText(cnt, 2, "+");
-                        else if (progRecPriority < 0)
-                            ltype->SetItemText(cnt, 2, "-");
-                        ltype->SetItemText(cnt, 3, 
-                                        QString::number(abs(progRecPriority)));
+                        if (progRecPriority < 0)
+                            ltype->SetItemText(cnt, 3, "-");
+                        else
+                            ltype->SetItemText(cnt, 3, "+");
+                        ltype->SetItemText(cnt, 4, 
+                                QString::number(abs(progRecPriority)));
 
-                        if (finalRecPriority >= 0)
-                            ltype->SetItemText(cnt, 4, "+");
-                        else if (finalRecPriority < 0)
-                            ltype->SetItemText(cnt, 4, "-");
+                        if (finalRecPriority < 0)
+                            ltype->SetItemText(cnt, 5, "-");
+                        else
+                            ltype->SetItemText(cnt, 5, "+");
 
-                        ltype->SetItemText(cnt, 5, 
-                                           QString::number(abs(finalRecPriority)));
+                        ltype->SetItemText(cnt, 6, 
+                                QString::number(abs(finalRecPriority)));
 
-                        if (progInfo->rectype == kDontRecord)
+                        if (progInfo->recType == kDontRecord ||
+                            progInfo->recstatus == rsInactive)
                             ltype->EnableForcedFont(cnt, "inactive");
+                        else if (recMatch[progInfo->recordid] > 0)
+                            ltype->EnableForcedFont(cnt, "recording");
+                        // else if (listMatch[progInfo->recordid] < 1)
+                        //    ltype->EnableForcedFont(cnt, "dormant");
 
                         cnt++;
                         listCount++;
@@ -914,10 +1092,23 @@ void ProgramRecPriority::updateInfo(QPainter *p)
         rectype = curitem->recType;
 
         QString subtitle = "";
-        if (curitem->subtitle != "(null)")
+        if (curitem->subtitle != "(null)" &&
+            (curitem->rectype == kSingleRecord ||
+             curitem->rectype == kOverrideRecord ||
+             curitem->rectype == kDontRecord))
+        {
             subtitle = curitem->subtitle;
+        }
+
+        QString matchInfo;
+        if (curitem->recstatus == rsInactive)
+            matchInfo = curitem->RecStatusText();
         else
-            subtitle = "";
+            matchInfo = QString(tr("Recording %1 of %2"))
+                                   .arg(recMatch[curitem->recordid])
+                                   .arg(listMatch[curitem->recordid]);
+
+        subtitle = QString("(%1) %2").arg(matchInfo).arg(subtitle);
 
         LayerSet *container = NULL;
         container = theme->GetSet("program_info");
@@ -940,25 +1131,31 @@ void ProgramRecPriority::updateInfo(QPainter *p)
                         text = tr("Recording just this showing");
                         break;
                     case kOverrideRecord:
-                        text = tr("Recording this showing with override options");
+                        text = tr("Recording with override options");
                         break;
                     case kWeekslotRecord:
                         text = tr("Recording every week");
                         break;
                     case kTimeslotRecord:
-                        text = tr("Recording when shown in this timeslot");
+                        text = tr("Recording in this timeslot");
                         break;
                     case kChannelRecord:
-                        text = tr("Recording when shown on this channel");
+                        text = tr("Recording on this channel");
                         break;
                     case kAllRecord:
                         text = tr("Recording all showings");
                         break;
                     case kFindOneRecord:
-                        text = tr("Recording one showing of this program");
+                        text = tr("Recording one showing");
+                        break;
+                    case kFindDailyRecord:
+                        text = tr("Recording a showing daily");
+                        break;
+                    case kFindWeeklyRecord:
+                        text = tr("Recording a showing weekly");
                         break;
                     case kDontRecord:
-                        text = tr("Manually not recording this showing");
+                        text = tr("Not allowed to record this showing");
                         break;
                     case kNotRecording:
                         text = tr("Not recording this showing");
@@ -984,7 +1181,8 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("channel");
             if (type) {
-                if (rectype != kAllRecord && rectype != kFindOneRecord)
+                if (rectype != kAllRecord && rectype != kFindOneRecord &&
+                    rectype != kFindDailyRecord && rectype != kFindWeeklyRecord)
                     type->SetText(curitem->ChannelText(channelFormat));
                 else
                     type->SetText(tr("Any"));
@@ -992,7 +1190,8 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("longchannel");
             if (type) {
-                if (rectype != kAllRecord && rectype != kFindOneRecord)
+                if (rectype != kAllRecord && rectype != kFindOneRecord &&
+                    rectype != kFindDailyRecord && rectype != kFindWeeklyRecord)
                     type->SetText(curitem->ChannelText(longChannelFormat));
                 else
                     type->SetText(tr("Any"));
