@@ -16,6 +16,8 @@
 #include <iostream>
 
 #include "libmyth/mythcontext.h"
+#include "libmyth/langsettings.h"
+#include "libmyth/dialogbox.h"
 #include "libmythtv/videosource.h"
 #include "libmythtv/channeleditor.h"
 #include "libmyth/themedmenu.h"
@@ -25,29 +27,8 @@
 
 using namespace std;
 
-QTranslator *translator;
 QSqlDatabase* db;
 
-QString getResponse(const QString &query, const QString &def)
-{
-    cout << query;
-
-    if (def != "")
-    {
-        cout << " [" << def << "]  ";
-    }
-    
-    char response[80];
-    cin.getline(response, 80);
-
-    QString qresponse = response;
-
-    if (qresponse == "")
-        qresponse = def;
-
-    return qresponse;
-}
-
 void clearCardDB(void)
 {
     QSqlQuery query;
@@ -117,10 +98,14 @@ void SetupMenu(void)
 
 int main(int argc, char *argv[])
 {
+#ifdef Q_WS_MACX
+    // Without this, we can't set focus to any of the CheckBoxSetting, and most
+    // of the MythPushButton widgets, and they don't use the themed background.
+    QApplication::setDesktopSettingsAware(FALSE);
+#endif
     QApplication a(argc, argv);
 
     gContext = new MythContext(MYTH_BINARY_VERSION, true);
-    translator = new QTranslator(0);
 
     db = QSqlDatabase::addDatabase("QMYSQL3");
     if (!gContext->OpenDatabase(db))
@@ -136,10 +121,7 @@ int main(int argc, char *argv[])
 
     UpgradeTVDatabaseSchema();
 
-    translator->load(PREFIX + QString("/share/mythtv/i18n/mythfrontend_") + 
-                     QString(gContext->GetSetting("Language").lower()) + 
-                     QString(".qm"), ".");
-    a.installTranslator(translator);
+    LanguageSettings::load("mythfrontend");
 
     gContext->SetSetting("Theme", "blue");
     gContext->LoadQtConfig();
@@ -150,23 +132,34 @@ int main(int argc, char *argv[])
     if (!dir.exists())
         dir.mkdir(fileprefix);
 
-    QString response = getResponse("Would you like to clear all capture card\n"
-                                   "settings before starting configuration?", 
-                                   "no");
-
-    if (response.left(1).lower() == "y")
-        clearCardDB();
-
-    response = getResponse("Would you like to clear all program/channel\n"
-                           "settings before starting configuration?",
-                           "no");
-
-    if (response.left(1).lower() == "y")
-        clearAllDB();
-
     MythMainWindow *mainWindow = new MythMainWindow();
     mainWindow->Show();
     gContext->SetMainWindow(mainWindow);
+    LanguageSettings::prompt();
+
+    DialogBox dboxCard(mainWindow, QObject::tr("Would you like to clear all "
+                                   "capture card settings before starting "
+                                   "configuration?"));
+    dboxCard.AddButton(QObject::tr("No, leave my card settings alone"));
+    dboxCard.AddButton(QObject::tr("Yes, delete my card settings"));
+    if (dboxCard.exec() == 2)
+        clearCardDB();
+    
+    // Give the user time to realize the first dialog is gone
+    // before we bring up a similar-looking one
+    usleep(750000);
+    
+    DialogBox dboxProg(mainWindow, QObject::tr("Would you like to clear all "
+                                   "program data and channel settings before "
+                                   "starting configuration? This will not "
+                                   "affect any existing recordings."));
+    dboxProg.AddButton(QObject::tr("No, leave my channel settings alone"));
+    dboxProg.AddButton(QObject::tr("Yes, delete my channel settings"));
+    if (dboxProg.exec() == 2)
+        clearAllDB();
+
+    REG_KEY("qt", "DELETE", "Delete", "D");
+    REG_KEY("qt", "EDIT", "Edit", "E");
 
     SetupMenu();
 
